@@ -1,11 +1,12 @@
+use std::collections::HashMap;
 use std::sync::atomic::{AtomicBool, Ordering};
 use std::sync::{Arc, Mutex};
 
+use zellij_server::panes::sixel::SixelImageStore;
 use zellij_server::panes::{LinkHandler, TerminalPane};
-use zellij_utils::pane_size::{Dimension, PaneGeom, Size};
+use zellij_utils::data::{Palette, Style};
+use zellij_utils::pane_size::{Dimension, PaneGeom, Size, SizeInPixels};
 use zellij_utils::vte;
-use zellij_utils::zellij_tile::data::Palette;
-use zellij_utils::zellij_tile::prelude::Style;
 
 use ssh2::Session;
 use std::io::prelude::*;
@@ -19,7 +20,9 @@ use std::rc::Rc;
 const ZELLIJ_EXECUTABLE_LOCATION: &str = "/usr/src/zellij/x86_64-unknown-linux-musl/release/zellij";
 const SET_ENV_VARIABLES: &str = "EDITOR=/usr/bin/vi";
 const ZELLIJ_LAYOUT_PATH: &str = "/usr/src/zellij/fixtures/layouts";
+const ZELLIJ_CONFIG_PATH: &str = "/usr/src/zellij/fixtures/configs";
 const ZELLIJ_DATA_DIR: &str = "/usr/src/zellij/e2e-data";
+const ZELLIJ_FIXTURE_PATH: &str = "/usr/src/zellij/fixtures";
 const CONNECTION_STRING: &str = "127.0.0.1:2222";
 const CONNECTION_USERNAME: &str = "test";
 const CONNECTION_PASSWORD: &str = "test";
@@ -76,6 +79,7 @@ fn start_zellij(channel: &mut ssh2::Channel) {
         )
         .unwrap();
     channel.flush().unwrap();
+    std::thread::sleep(std::time::Duration::from_secs(1)); // wait until Zellij stops parsing startup ANSI codes from the terminal STDIN
 }
 
 fn start_zellij_mirrored_session(channel: &mut ssh2::Channel) {
@@ -90,6 +94,7 @@ fn start_zellij_mirrored_session(channel: &mut ssh2::Channel) {
         )
         .unwrap();
     channel.flush().unwrap();
+    std::thread::sleep(std::time::Duration::from_secs(1)); // wait until Zellij stops parsing startup ANSI codes from the terminal STDIN
 }
 
 fn start_zellij_in_session(channel: &mut ssh2::Channel, session_name: &str, mirrored: bool) {
@@ -108,6 +113,7 @@ fn start_zellij_in_session(channel: &mut ssh2::Channel, session_name: &str, mirr
         )
         .unwrap();
     channel.flush().unwrap();
+    std::thread::sleep(std::time::Duration::from_secs(1)); // wait until Zellij stops parsing startup ANSI codes from the terminal STDIN
 }
 
 fn attach_to_existing_session(channel: &mut ssh2::Channel, session_name: &str) {
@@ -121,6 +127,7 @@ fn attach_to_existing_session(channel: &mut ssh2::Channel, session_name: &str) {
         )
         .unwrap();
     channel.flush().unwrap();
+    std::thread::sleep(std::time::Duration::from_secs(1)); // wait until Zellij stops parsing startup ANSI codes from the terminal STDIN
 }
 
 fn start_zellij_without_frames(channel: &mut ssh2::Channel) {
@@ -135,6 +142,7 @@ fn start_zellij_without_frames(channel: &mut ssh2::Channel) {
         )
         .unwrap();
     channel.flush().unwrap();
+    std::thread::sleep(std::time::Duration::from_secs(1)); // wait until Zellij stops parsing startup ANSI codes from the terminal STDIN
 }
 
 fn start_zellij_with_layout(channel: &mut ssh2::Channel, layout_path: &str) {
@@ -153,6 +161,26 @@ fn start_zellij_with_layout(channel: &mut ssh2::Channel, layout_path: &str) {
         )
         .unwrap();
     channel.flush().unwrap();
+    std::thread::sleep(std::time::Duration::from_secs(1)); // wait until Zellij stops parsing startup ANSI codes from the terminal STDIN
+}
+
+fn start_zellij_with_config(channel: &mut ssh2::Channel, config_path: &str) {
+    stop_zellij(channel);
+    channel
+        .write_all(
+            format!(
+                "{} {} --config {} --session {} --data-dir {}\n",
+                SET_ENV_VARIABLES,
+                ZELLIJ_EXECUTABLE_LOCATION,
+                config_path,
+                SESSION_NAME,
+                ZELLIJ_DATA_DIR
+            )
+            .as_bytes(),
+        )
+        .unwrap();
+    channel.flush().unwrap();
+    std::thread::sleep(std::time::Duration::from_secs(1)); // wait until Zellij stops parsing startup ANSI codes from the terminal STDIN
 }
 
 fn read_from_channel(
@@ -174,6 +202,11 @@ fn read_from_channel(
                 let mut retries_left = 3;
                 let mut should_sleep = false;
                 let mut vte_parser = vte::Parser::new();
+                let character_cell_size = Rc::new(RefCell::new(Some(SizeInPixels {
+                    height: 21,
+                    width: 8,
+                })));
+                let sixel_image_store = Rc::new(RefCell::new(SixelImageStore::default()));
                 let mut terminal_output = TerminalPane::new(
                     0,
                     pane_geom,
@@ -181,8 +214,10 @@ fn read_from_channel(
                     0,
                     String::new(),
                     Rc::new(RefCell::new(LinkHandler::new())),
-                    Rc::new(RefCell::new(None)),
+                    character_cell_size,
+                    sixel_image_store,
                     Rc::new(RefCell::new(Palette::default())),
+                    Rc::new(RefCell::new(HashMap::new())),
                 ); // 0 is the pane index
                 loop {
                     if !should_keep_running.load(Ordering::SeqCst) {
@@ -201,7 +236,7 @@ fn read_from_channel(
                                 terminal_output.cursor_coordinates().unwrap_or((0, 0));
                             *last_snapshot = current_snapshot;
                             should_sleep = true;
-                        }
+                        },
                         Ok(count) => {
                             for byte in buf.iter().take(count) {
                                 vte_parser.advance(&mut terminal_output.grid, *byte);
@@ -212,7 +247,7 @@ fn read_from_channel(
                                 terminal_output.grid.cursor_coordinates().unwrap_or((0, 0));
                             *last_snapshot = current_snapshot;
                             should_sleep = true;
-                        }
+                        },
                         Err(e) => {
                             if e.kind() == std::io::ErrorKind::WouldBlock {
                                 let current_snapshot = take_snapshot(&mut terminal_output);
@@ -226,7 +261,7 @@ fn read_from_channel(
                             } else {
                                 break;
                             }
-                        }
+                        },
                     }
                 }
             }
@@ -322,6 +357,14 @@ impl RemoteTerminal {
             )
             .unwrap();
         channel.flush().unwrap();
+        std::thread::sleep(std::time::Duration::from_secs(1)); // wait until Zellij stops parsing startup ANSI codes from the terminal STDIN
+    }
+    pub fn load_fixture(&mut self, name: &str) {
+        let mut channel = self.channel.lock().unwrap();
+        channel
+            .write_all(format!("cat {ZELLIJ_FIXTURE_PATH}/{name}\n").as_bytes())
+            .unwrap();
+        channel.flush().unwrap();
     }
 }
 
@@ -564,10 +607,47 @@ impl RemoteRunner {
             reader_thread,
         }
     }
+    pub fn new_with_config(win_size: Size, config_file_name: &'static str) -> Self {
+        let remote_path = Path::new(ZELLIJ_CONFIG_PATH).join(config_file_name);
+        let sess = ssh_connect();
+        let mut channel = sess.channel_session().unwrap();
+        let mut rows = Dimension::fixed(win_size.rows);
+        let mut cols = Dimension::fixed(win_size.cols);
+        rows.set_inner(win_size.rows);
+        cols.set_inner(win_size.cols);
+        let pane_geom = PaneGeom {
+            x: 0,
+            y: 0,
+            rows,
+            cols,
+        };
+        setup_remote_environment(&mut channel, win_size);
+        start_zellij_with_config(&mut channel, &remote_path.to_string_lossy());
+        let channel = Arc::new(Mutex::new(channel));
+        let last_snapshot = Arc::new(Mutex::new(String::new()));
+        let cursor_coordinates = Arc::new(Mutex::new((0, 0)));
+        sess.set_blocking(false);
+        let reader_thread =
+            read_from_channel(&channel, &last_snapshot, &cursor_coordinates, &pane_geom);
+        RemoteRunner {
+            steps: vec![],
+            channel,
+            currently_running_step: None,
+            current_step_index: 0,
+            retries_left: RETRIES,
+            retry_pause_ms: 100,
+            test_timed_out: false,
+            panic_on_no_retries_left: true,
+            last_snapshot,
+            cursor_coordinates,
+            reader_thread,
+        }
+    }
     pub fn dont_panic(mut self) -> Self {
         self.panic_on_no_retries_left = false;
         self
     }
+    #[allow(unused)]
     pub fn retry_pause_ms(mut self, retry_pause_ms: usize) -> Self {
         self.retry_pause_ms = retry_pause_ms;
         self
