@@ -11,7 +11,7 @@ use std::path::PathBuf;
 use std::process::{Child, Command};
 use std::sync::{Arc, Mutex};
 
-use zellij_utils::{async_std, interprocess, libc, nix, signal_hook, zellij_tile};
+use zellij_utils::{async_std, interprocess, libc, nix, signal_hook};
 
 use async_std::fs::File as AsyncFile;
 use async_std::os::unix::io::FromRawFd;
@@ -25,8 +25,8 @@ use nix::sys::termios;
 
 use nix::unistd;
 use signal_hook::consts::*;
-use zellij_tile::data::Palette;
 use zellij_utils::{
+    data::Palette,
     input::command::{RunCommand, TerminalAction},
     ipc::{ClientToServerMsg, IpcReceiverWithContext, IpcSenderWithContext, ServerToClientMsg},
     shared::default_palette,
@@ -73,10 +73,10 @@ fn handle_command_exit(mut child: Child) {
                 // and exit this function
                 // TODO: handle errors?
                 break 'handle_exit;
-            }
+            },
             Ok(None) => {
                 ::std::thread::sleep(::std::time::Duration::from_millis(10));
-            }
+            },
             Err(e) => panic!("error attempting to wait: {}", e),
         }
 
@@ -157,11 +157,29 @@ fn handle_terminal(
             Some(failover_cmd) => handle_terminal(failover_cmd, None, orig_termios, quit_cb),
             None => {
                 panic!("failed to start pty{:?}", e);
-            }
+            },
         },
     }
 }
 
+// this is a utility method to separate the arguments from a pathbuf before we turn it into a
+// Command. eg. "/usr/bin/vim -e" ==> "/usr/bin/vim" + "-e" (the latter will be pushed to args)
+fn separate_command_arguments(command: &mut PathBuf, args: &mut Vec<String>) {
+    if let Some(file_name) = command
+        .file_name()
+        .and_then(|f_n| f_n.to_str())
+        .map(|f_n| f_n.to_string())
+    {
+        let mut file_name_parts = file_name.split_ascii_whitespace();
+        if let Some(first_part) = file_name_parts.next() {
+            command.set_file_name(first_part);
+            for part in file_name_parts {
+                args.push(String::from(part));
+            }
+        }
+    }
+}
+
 /// If a [`TerminalAction::OpenFile(file)`] is given, the text editor specified by environment variable `EDITOR`
 /// (or `VISUAL`, if `EDITOR` is not set) will be started in the new terminal, with the given
 /// file open.
@@ -191,11 +209,16 @@ pub fn spawn_terminal(
                     "No Editor found, consider setting a path to one in $EDITOR or $VISUAL",
                 );
             }
-            let command = default_editor.unwrap_or_else(|| {
+
+            let mut command = default_editor.unwrap_or_else(|| {
                 PathBuf::from(env::var("EDITOR").unwrap_or_else(|_| env::var("VISUAL").unwrap()))
             });
 
             let mut args = vec![];
+
+            if !command.is_dir() {
+                separate_command_arguments(&mut command, &mut args);
+            }
             let file_to_open = file_to_open
                 .into_os_string()
                 .into_string()
@@ -217,7 +240,7 @@ pub fn spawn_terminal(
                 args,
                 cwd: None,
             }
-        }
+        },
         TerminalAction::RunCommand(command) => command,
     };
     let failover_cmd = if let Some(failover_cmd_args) = failover_cmd_args {
@@ -271,7 +294,7 @@ pub trait ServerOsApi: Send + Sync {
     /// Sets the size of the terminal associated to file descriptor `fd`.
     fn set_terminal_size_using_fd(&self, fd: RawFd, cols: u16, rows: u16);
     /// Spawn a new terminal, with a terminal action. The returned tuple contains the master file
-    /// descriptor of the forked psuedo terminal and a [ChildId] struct containing process id's for
+    /// descriptor of the forked pseudo terminal and a [ChildId] struct containing process id's for
     /// the forked child process.
     fn spawn_terminal(
         &self,
@@ -389,11 +412,10 @@ impl ServerOsApi for ServerOsInputOutput {
         None
     }
     fn write_to_file(&mut self, buf: String, name: Option<String>) {
-        let mut f: File;
-        match name {
-            Some(x) => f = File::create(x).unwrap(),
-            None => f = tempfile().unwrap(),
-        }
+        let mut f: File = match name {
+            Some(x) => File::create(x).unwrap(),
+            None => tempfile().unwrap(),
+        };
         if let Err(e) = write!(f, "{}", buf) {
             log::error!("could not write to file: {}", e);
         }
