@@ -1,4 +1,5 @@
-use crate::output::CharacterChunk;
+use crate::output::{CharacterChunk, SixelImageChunk};
+use crate::panes::sixel::SixelImageStore;
 use crate::panes::{
     grid::Grid,
     terminal_character::{CursorShape, TerminalCharacter, EMPTY_TERMINAL_CHARACTER},
@@ -13,15 +14,14 @@ use std::fmt::Debug;
 use std::os::unix::io::RawFd;
 use std::rc::Rc;
 use std::time::{self, Instant};
-use zellij_tile::prelude::Style;
 use zellij_utils::pane_size::Offset;
 use zellij_utils::{
+    data::{InputMode, Palette, PaletteColor, Style},
     pane_size::SizeInPixels,
     pane_size::{Dimension, PaneGeom},
     position::Position,
     shared::make_terminal_title,
     vte,
-    zellij_tile::data::{InputMode, Palette, PaletteColor},
 };
 
 pub const SELECTION_SCROLL_INTERVAL_MS: u64 = 10;
@@ -36,6 +36,7 @@ pub enum PaneId {
 
 // FIXME: This should hold an os_api handle so that terminal panes can set their own size via FD in
 // their `reflow_lines()` method. Drop a Box<dyn ServerOsApi> in here somewhere.
+#[allow(clippy::too_many_arguments)]
 pub struct TerminalPane {
     pub grid: Grid,
     pub pid: RawFd,
@@ -49,9 +50,11 @@ pub struct TerminalPane {
     content_offset: Offset,
     pane_title: String,
     pane_name: String,
+    prev_pane_name: String,
     frame: HashMap<ClientId, PaneFrame>,
     borderless: bool,
     fake_cursor_locations: HashSet<(usize, usize)>, // (x, y) - these hold a record of previous fake cursors which we need to clear on render
+    search_term: String,
 }
 
 impl Pane for TerminalPane {
@@ -93,15 +96,15 @@ impl Pane for TerminalPane {
         self.geom = position_and_size;
         self.reflow_lines();
     }
-    fn get_geom_override(&mut self, pane_geom: PaneGeom) {
+    fn set_geom_override(&mut self, pane_geom: PaneGeom) {
         self.geom_override = Some(pane_geom);
         self.reflow_lines();
     }
     fn handle_pty_bytes(&mut self, bytes: VteBytes) {
+        self.set_should_render(true);
         for &byte in &bytes {
             self.vte_parser.advance(&mut self.grid, byte);
         }
-        self.set_should_render(true);
     }
     fn cursor_coordinates(&self) -> Option<(usize, usize)> {
         // (x, y)
@@ -123,7 +126,7 @@ impl Pane for TerminalPane {
                     // some editors will not show this
                     return "OD".as_bytes().to_vec();
                 }
-            }
+            },
             [27, 91, 67] => {
                 // right arrow
                 if self.grid.cursor_key_mode {
@@ -131,7 +134,7 @@ impl Pane for TerminalPane {
                     // some editors will not show this
                     return "OC".as_bytes().to_vec();
                 }
-            }
+            },
             [27, 91, 65] => {
                 // up arrow
                 if self.grid.cursor_key_mode {
@@ -139,20 +142,20 @@ impl Pane for TerminalPane {
                     // some editors will not show this
                     return "OA".as_bytes().to_vec();
                 }
-            }
+            },
 
             [27, 91, 72] => {
                 // home key
                 if self.grid.cursor_key_mode {
                     return vec![27, 79, 72]; // ESC O H
                 }
-            }
+            },
             [27, 91, 70] => {
                 // end key
                 if self.grid.cursor_key_mode {
                     return vec![27, 79, 70]; // ESC O F
                 }
-            }
+            },
             [27, 91, 66] => {
                 // down arrow
                 if self.grid.cursor_key_mode {
@@ -160,7 +163,7 @@ impl Pane for TerminalPane {
                     // some editors will not show this
                     return "OB".as_bytes().to_vec();
                 }
-            }
+            },
             [27, 91, 50, 48, 48, 126] | [27, 91, 50, 48, 49, 126] => {
                 if !self.grid.bracketed_paste_mode {
                     // Zellij itself operates in bracketed paste mode, so the terminal sends these
@@ -169,8 +172,8 @@ impl Pane for TerminalPane {
                     // panes who do not work in this mode
                     return vec![];
                 }
-            }
-            _ => {}
+            },
+            _ => {},
         };
         input_bytes
     }
@@ -204,13 +207,14 @@ impl Pane for TerminalPane {
     fn render(
         &mut self,
         _client_id: Option<ClientId>,
-    ) -> Option<(Vec<CharacterChunk>, Option<String>)> {
+    ) -> Option<(Vec<CharacterChunk>, Option<String>, Vec<SixelImageChunk>)> {
         if self.should_render() {
             let mut raw_vte_output = String::new();
             let content_x = self.get_content_x();
             let content_y = self.get_content_y();
 
-            let mut character_chunks = self.grid.read_changes(content_x, content_y);
+            let (mut character_chunks, sixel_image_chunks) =
+                self.grid.read_changes(content_x, content_y);
             for character_chunk in character_chunks.iter_mut() {
                 character_chunk.add_changed_colors(self.grid.changed_colors);
                 if self
@@ -222,12 +226,39 @@ impl Pane for TerminalPane {
                         PaletteColor::Rgb(rgb) => AnsiCode::RgbCode(rgb),
                         PaletteColor::EightBit(col) => AnsiCode::ColorIndex(col),
                     };
-                    character_chunk.add_selection_and_background(
+                    character_chunk.add_selection_and_colors(
                         self.grid.selection,
                         background_color,
+                        None,
                         content_x,
                         content_y,
                     );
+                } else if !self.grid.search_results.selections.is_empty() {
+                    for res in self.grid.search_results.selections.iter() {
+                        if res.contains_row(character_chunk.y.saturating_sub(content_y)) {
+                            let (select_background_palette, select_foreground_palette) =
+                                if Some(res) == self.grid.search_results.active.as_ref() {
+                                    (self.style.colors.orange, self.style.colors.black)
+                                } else {
+                                    (self.style.colors.green, self.style.colors.black)
+                                };
+                            let background_color = match select_background_palette {
+                                PaletteColor::Rgb(rgb) => AnsiCode::RgbCode(rgb),
+                                PaletteColor::EightBit(col) => AnsiCode::ColorIndex(col),
+                            };
+                            let foreground_color = match select_foreground_palette {
+                                PaletteColor::Rgb(rgb) => AnsiCode::RgbCode(rgb),
+                                PaletteColor::EightBit(col) => AnsiCode::ColorIndex(col),
+                            };
+                            character_chunk.add_selection_and_colors(
+                                *res,
+                                background_color,
+                                Some(foreground_color),
+                                content_x,
+                                content_y,
+                            );
+                        }
+                    }
                 }
             }
             if self.grid.ring_bell {
@@ -236,7 +267,7 @@ impl Pane for TerminalPane {
                 self.grid.ring_bell = false;
             }
             self.set_should_render(false);
-            Some((character_chunks, Some(raw_vte_output)))
+            Some((character_chunks, Some(raw_vte_output), sixel_image_chunks))
         } else {
             None
         }
@@ -253,6 +284,31 @@ impl Pane for TerminalPane {
             && frame_params.is_main_client
         {
             String::from("Enter name...")
+        } else if input_mode == InputMode::EnterSearch
+            && frame_params.is_main_client
+            && self.search_term.is_empty()
+        {
+            String::from("Enter search...")
+        } else if (input_mode == InputMode::EnterSearch || input_mode == InputMode::Search)
+            && !self.search_term.is_empty()
+        {
+            let mut modifier_text = String::new();
+            if self.grid.search_results.has_modifiers_set() {
+                let mut modifiers = Vec::new();
+                modifier_text.push_str(" [");
+                if self.grid.search_results.case_insensitive {
+                    modifiers.push("c")
+                }
+                if self.grid.search_results.whole_word_only {
+                    modifiers.push("o")
+                }
+                if self.grid.search_results.wrap_search {
+                    modifiers.push("w")
+                }
+                modifier_text.push_str(&modifiers.join(", "));
+                modifier_text.push(']');
+            }
+            format!("SEARCHING: {}{}", self.search_term, modifier_text)
         } else if self.pane_name.is_empty() {
             self.grid
                 .title
@@ -261,6 +317,7 @@ impl Pane for TerminalPane {
         } else {
             self.pane_name.clone()
         };
+
         let frame = PaneFrame::new(
             self.current_geom().into(),
             self.grid.scrollback_position_and_length(),
@@ -281,7 +338,7 @@ impl Pane for TerminalPane {
                 } else {
                     None
                 }
-            }
+            },
             None => {
                 if !self.borderless {
                     let frame_output = frame.render();
@@ -290,7 +347,7 @@ impl Pane for TerminalPane {
                 } else {
                     None
                 }
-            }
+            },
         }
     }
     fn render_fake_cursor(
@@ -333,14 +390,14 @@ impl Pane for TerminalPane {
         match name {
             "\0" => {
                 self.pane_name = String::new();
-            }
+            },
             "\u{007F}" | "\u{0008}" => {
                 //delete and backspace keys
                 self.pane_name.pop();
-            }
+            },
             c => {
                 self.pane_name.push_str(c);
-            }
+            },
         }
     }
     fn pid(&self) -> PaneId {
@@ -387,7 +444,7 @@ impl Pane for TerminalPane {
         self.reflow_lines();
     }
     fn dump_screen(&mut self, _client_id: ClientId) -> String {
-        return self.grid.dump_screen();
+        self.grid.dump_screen()
     }
     fn scroll_up(&mut self, count: usize, _client_id: ClientId) {
         self.grid.move_viewport_up(count);
@@ -471,6 +528,17 @@ impl Pane for TerminalPane {
         self.reflow_lines();
     }
 
+    fn store_pane_name(&mut self) {
+        if self.pane_name != self.prev_pane_name {
+            self.prev_pane_name = self.pane_name.clone()
+        }
+    }
+    fn load_pane_name(&mut self) {
+        if self.pane_name != self.prev_pane_name {
+            self.pane_name = self.prev_pane_name.clone()
+        }
+    }
+
     fn set_borderless(&mut self, borderless: bool) {
         self.borderless = borderless;
     }
@@ -481,13 +549,64 @@ impl Pane for TerminalPane {
     fn mouse_mode(&self) -> bool {
         self.grid.mouse_mode
     }
+
     fn get_line_number(&self) -> Option<usize> {
         // + 1 because the absolute position in the scrollback is 0 indexed and this should be 1 indexed
         Some(self.grid.absolute_position_in_scrollback() + 1)
     }
+
+    fn update_search_term(&mut self, needle: &str) {
+        match needle {
+            "\0" => {
+                self.search_term = String::new();
+            },
+            "\u{007F}" | "\u{0008}" => {
+                //delete and backspace keys
+                self.search_term.pop();
+            },
+            c => {
+                self.search_term.push_str(c);
+            },
+        }
+        self.grid.clear_search();
+        if !self.search_term.is_empty() {
+            self.grid.set_search_string(&self.search_term);
+        }
+        self.set_should_render(true);
+    }
+    fn search_down(&mut self) {
+        if self.search_term.is_empty() {
+            return; // No-op
+        }
+        self.grid.search_down();
+        self.set_should_render(true);
+    }
+    fn search_up(&mut self) {
+        if self.search_term.is_empty() {
+            return; // No-op
+        }
+        self.grid.search_up();
+        self.set_should_render(true);
+    }
+    fn toggle_search_case_sensitivity(&mut self) {
+        self.grid.toggle_search_case_sensitivity();
+        self.set_should_render(true);
+    }
+    fn toggle_search_whole_words(&mut self) {
+        self.grid.toggle_search_whole_words();
+        self.set_should_render(true);
+    }
+    fn toggle_search_wrap(&mut self) {
+        self.grid.toggle_search_wrap();
+    }
+    fn clear_search(&mut self) {
+        self.grid.clear_search();
+        self.search_term.clear();
+    }
 }
 
 impl TerminalPane {
+    #[allow(clippy::too_many_arguments)]
     pub fn new(
         pid: RawFd,
         position_and_size: PaneGeom,
@@ -496,15 +615,19 @@ impl TerminalPane {
         pane_name: String,
         link_handler: Rc<RefCell<LinkHandler>>,
         character_cell_size: Rc<RefCell<Option<SizeInPixels>>>,
+        sixel_image_store: Rc<RefCell<SixelImageStore>>,
         terminal_emulator_colors: Rc<RefCell<Palette>>,
+        terminal_emulator_color_codes: Rc<RefCell<HashMap<usize, String>>>,
     ) -> TerminalPane {
         let initial_pane_title = format!("Pane #{}", pane_index);
         let grid = Grid::new(
             position_and_size.rows.as_usize(),
             position_and_size.cols.as_usize(),
             terminal_emulator_colors,
+            terminal_emulator_color_codes,
             link_handler,
             character_cell_size,
+            sixel_image_store,
         );
         TerminalPane {
             frame: HashMap::new(),
@@ -519,9 +642,11 @@ impl TerminalPane {
             style,
             selection_scrolled_at: time::Instant::now(),
             pane_title: initial_pane_title,
-            pane_name,
+            pane_name: pane_name.clone(),
+            prev_pane_name: pane_name,
             borderless: false,
             fake_cursor_locations: HashSet::new(),
+            search_term: String::new(),
         }
     }
     pub fn get_x(&self) -> usize {
@@ -566,3 +691,7 @@ impl TerminalPane {
 #[cfg(test)]
 #[path = "./unit/terminal_pane_tests.rs"]
 mod grid_tests;
+
+#[cfg(test)]
+#[path = "./unit/search_in_pane_tests.rs"]
+mod search_tests;
