@@ -7,9 +7,8 @@ mod copy_command;
 use copy_command::CopyCommand;
 use std::env::temp_dir;
 use uuid::Uuid;
-use zellij_tile::prelude::Style;
 use zellij_utils::position::{Column, Line};
-use zellij_utils::{position::Position, serde, zellij_tile};
+use zellij_utils::{position::Position, serde};
 
 use crate::pty_writer::PtyWriteInstruction;
 use crate::screen::CopyOptions;
@@ -18,7 +17,8 @@ use crate::ui::pane_boundaries_frame::FrameParams;
 use self::clipboard::ClipboardProvider;
 use crate::{
     os_input_output::ServerOsApi,
-    output::{CharacterChunk, Output},
+    output::{CharacterChunk, Output, SixelImageChunk},
+    panes::sixel::SixelImageStore,
     panes::{FloatingPanes, TiledPanes},
     panes::{LinkHandler, PaneId, PluginPane, TerminalPane},
     pty::{ClientOrTabIndex, PtyInstruction, VteBytes},
@@ -36,8 +36,8 @@ use std::{
     collections::{HashMap, HashSet},
     str,
 };
-use zellij_tile::data::{Event, InputMode, ModeInfo, Palette, PaletteColor};
 use zellij_utils::{
+    data::{Event, InputMode, ModeInfo, Palette, PaletteColor, Style},
     input::{
         command::TerminalAction,
         layout::{Layout, Run},
@@ -70,6 +70,7 @@ pub(crate) struct Tab {
     pub index: usize,
     pub position: usize,
     pub name: String,
+    pub prev_name: String,
     tiled_panes: TiledPanes,
     floating_panes: FloatingPanes,
     suppressed_panes: HashMap<PaneId, Box<dyn Pane>>,
@@ -77,6 +78,7 @@ pub(crate) struct Tab {
     viewport: Rc<RefCell<Viewport>>, // includes all non-UI panes
     display_area: Rc<RefCell<Size>>, // includes all panes (including eg. the status bar and tab bar in the default layout)
     character_cell_size: Rc<RefCell<Option<SizeInPixels>>>,
+    sixel_image_store: Rc<RefCell<SixelImageStore>>,
     os_api: Box<dyn ServerOsApi>,
     pub senders: ThreadSenders,
     synchronize_is_active: bool,
@@ -96,6 +98,7 @@ pub(crate) struct Tab {
     copy_on_select: bool,
     last_mouse_hold_position: Option<Position>,
     terminal_emulator_colors: Rc<RefCell<Palette>>,
+    terminal_emulator_color_codes: Rc<RefCell<HashMap<usize, String>>>,
 }
 
 #[derive(Clone, Debug, Default, Serialize, Deserialize)]
@@ -120,7 +123,7 @@ pub trait Pane {
     fn get_content_rows(&self) -> usize;
     fn reset_size_and_position_override(&mut self);
     fn set_geom(&mut self, position_and_size: PaneGeom);
-    fn get_geom_override(&mut self, pane_geom: PaneGeom);
+    fn set_geom_override(&mut self, pane_geom: PaneGeom);
     fn handle_pty_bytes(&mut self, bytes: VteBytes);
     fn cursor_coordinates(&self) -> Option<(usize, usize)>;
     fn adjust_input_to_terminal(&self, input_bytes: Vec<u8>) -> Vec<u8>;
@@ -135,7 +138,7 @@ pub trait Pane {
     fn render(
         &mut self,
         client_id: Option<ClientId>,
-    ) -> Option<(Vec<CharacterChunk>, Option<String>)>; // TODO: better
+    ) -> Option<(Vec<CharacterChunk>, Option<String>, Vec<SixelImageChunk>)>; // TODO: better
     fn render_frame(
         &mut self,
         client_id: ClientId,
@@ -159,7 +162,7 @@ pub trait Pane {
     fn pull_left(&mut self, count: usize);
     fn pull_up(&mut self, count: usize);
     fn dump_screen(&mut self, _client_id: ClientId) -> String {
-        return "".to_owned();
+        "".to_owned()
     }
     fn scroll_up(&mut self, count: usize, client_id: ClientId);
     fn scroll_down(&mut self, count: usize, client_id: ClientId);
@@ -259,14 +262,35 @@ pub trait Pane {
     fn relative_position(&self, position_on_screen: &Position) -> Position {
         position_on_screen.relative_to(self.get_content_y(), self.get_content_x())
     }
-    fn position_is_on_frame(&self, position_on_screen: &Position) -> bool {
-        // TODO: handle cases where we have no frame
-        position_on_screen.line() == self.y() as isize
-            || position_on_screen.line()
-                == (self.y() as isize + self.rows() as isize).saturating_sub(1)
-            || position_on_screen.column() == self.x()
-            || position_on_screen.column() == (self.x() + self.cols()).saturating_sub(1)
+    fn position_is_on_frame(&self, position: &Position) -> bool {
+        if !self.contains(position) {
+            return false;
+        }
+        if (self.x()..self.get_content_x()).contains(&position.column()) {
+            // position is on left border
+            return true;
+        }
+        if (self.get_content_x() + self.get_content_columns()..(self.x() + self.cols()))
+            .contains(&position.column())
+        {
+            // position is on right border
+            return true;
+        }
+        if (self.y() as isize..self.get_content_y() as isize).contains(&position.line()) {
+            // position is on top border
+            return true;
+        }
+        if ((self.get_content_y() + self.get_content_rows()) as isize
+            ..(self.y() + self.rows()) as isize)
+            .contains(&position.line())
+        {
+            // position is on bottom border
+            return true;
+        }
+        false
     }
+    fn store_pane_name(&mut self);
+    fn load_pane_name(&mut self);
     fn set_borderless(&mut self, borderless: bool);
     fn borderless(&self) -> bool;
     fn handle_right_click(&mut self, _to: &Position, _client_id: ClientId) {}
@@ -274,6 +298,27 @@ pub trait Pane {
     fn get_line_number(&self) -> Option<usize> {
         None
     }
+    fn update_search_term(&mut self, _needle: &str) {
+        // No-op by default (only terminal-panes currently have search capability)
+    }
+    fn search_down(&mut self) {
+        // No-op by default (only terminal-panes currently have search capability)
+    }
+    fn search_up(&mut self) {
+        // No-op by default (only terminal-panes currently have search capability)
+    }
+    fn toggle_search_case_sensitivity(&mut self) {
+        // No-op by default (only terminal-panes currently have search capability)
+    }
+    fn toggle_search_whole_words(&mut self) {
+        // No-op by default (only terminal-panes currently have search capability)
+    }
+    fn toggle_search_wrap(&mut self) {
+        // No-op by default (only terminal-panes currently have search capability)
+    }
+    fn clear_search(&mut self) {
+        // No-op by default (only terminal-panes currently have search capability)
+    }
 }
 
 impl Tab {
@@ -285,6 +330,7 @@ impl Tab {
         name: String,
         display_area: Size,
         character_cell_size: Rc<RefCell<Option<SizeInPixels>>>,
+        sixel_image_store: Rc<RefCell<SixelImageStore>>,
         os_api: Box<dyn ServerOsApi>,
         senders: ThreadSenders,
         max_panes: Option<usize>,
@@ -296,6 +342,7 @@ impl Tab {
         client_id: ClientId,
         copy_options: CopyOptions,
         terminal_emulator_colors: Rc<RefCell<Palette>>,
+        terminal_emulator_color_codes: Rc<RefCell<HashMap<usize, String>>>,
     ) -> Self {
         let name = if name.is_empty() {
             format!("Tab #{}", index + 1)
@@ -346,11 +393,13 @@ impl Tab {
             tiled_panes,
             floating_panes,
             suppressed_panes: HashMap::new(),
-            name,
+            name: name.clone(),
+            prev_name: name,
             max_panes,
             viewport,
             display_area,
             character_cell_size,
+            sixel_image_store,
             synchronize_is_active: false,
             os_api,
             senders,
@@ -368,6 +417,7 @@ impl Tab {
             copy_on_select: copy_options.copy_on_select,
             last_mouse_hold_position: None,
             terminal_emulator_colors,
+            terminal_emulator_color_codes,
         }
     }
 
@@ -435,7 +485,9 @@ impl Tab {
                     layout.pane_name.clone().unwrap_or_default(),
                     self.link_handler.clone(),
                     self.character_cell_size.clone(),
+                    self.sixel_image_store.clone(),
                     self.terminal_emulator_colors.clone(),
+                    self.terminal_emulator_color_codes.clone(),
                 );
                 new_pane.set_borderless(layout.borderless);
                 self.tiled_panes
@@ -474,11 +526,11 @@ impl Tab {
             match next_selectable_pane_id {
                 Some(active_pane_id) => {
                     self.tiled_panes.focus_pane(active_pane_id, client_id);
-                }
+                },
                 None => {
                     // this is very likely a configuration error (layout with no selectable panes)
                     self.tiled_panes.clear_active_panes();
-                }
+                },
             }
         }
     }
@@ -628,7 +680,7 @@ impl Tab {
                         self.floating_panes
                             .focus_pane(first_floating_pane_id, client_id);
                     }
-                }
+                },
                 None => {
                     // there aren't any floating panes, we need to open a new one
                     //
@@ -645,7 +697,7 @@ impl Tab {
                         ClientOrTabIndex::ClientId(client_id),
                     );
                     self.senders.send_to_pty(instruction).unwrap();
-                }
+                },
             }
             self.floating_panes.set_force_render();
         }
@@ -665,7 +717,9 @@ impl Tab {
                         String::new(),
                         self.link_handler.clone(),
                         self.character_cell_size.clone(),
+                        self.sixel_image_store.clone(),
                         self.terminal_emulator_colors.clone(),
+                        self.terminal_emulator_color_codes.clone(),
                     );
                     new_pane.set_content_offset(Offset::frame(1)); // floating panes always have a frame
                     resize_pty!(new_pane, self.os_api);
@@ -688,7 +742,9 @@ impl Tab {
                         String::new(),
                         self.link_handler.clone(),
                         self.character_cell_size.clone(),
+                        self.sixel_image_store.clone(),
                         self.terminal_emulator_colors.clone(),
+                        self.terminal_emulator_color_codes.clone(),
                     );
                     self.tiled_panes.insert_pane(pid, Box::new(new_terminal));
                     self.should_clear_display_before_rendering = true;
@@ -714,7 +770,9 @@ impl Tab {
                     String::new(),
                     self.link_handler.clone(),
                     self.character_cell_size.clone(),
+                    self.sixel_image_store.clone(),
                     self.terminal_emulator_colors.clone(),
+                    self.terminal_emulator_color_codes.clone(),
                 );
                 let replaced_pane = if self.floating_panes.panes_are_visible() {
                     self.floating_panes
@@ -729,15 +787,15 @@ impl Tab {
                             .insert(PaneId::Terminal(pid), replaced_pane);
                         let current_active_pane = self.get_active_pane(client_id).unwrap(); // this will be the newly replaced pane we just created
                         resize_pty!(current_active_pane, self.os_api);
-                    }
+                    },
                     None => {
                         log::error!("Could not find editor pane to replace - is no pane focused?")
-                    }
+                    },
                 }
-            }
+            },
             PaneId::Plugin(_pid) => {
                 // TBD, currently unsupported
-            }
+            },
         }
     }
     pub fn horizontal_split(&mut self, pid: PaneId, client_id: ClientId) {
@@ -759,7 +817,9 @@ impl Tab {
                     String::new(),
                     self.link_handler.clone(),
                     self.character_cell_size.clone(),
+                    self.sixel_image_store.clone(),
                     self.terminal_emulator_colors.clone(),
+                    self.terminal_emulator_color_codes.clone(),
                 );
                 self.tiled_panes
                     .split_pane_horizontally(pid, Box::new(new_terminal), client_id);
@@ -787,7 +847,9 @@ impl Tab {
                     String::new(),
                     self.link_handler.clone(),
                     self.character_cell_size.clone(),
+                    self.sixel_image_store.clone(),
                     self.terminal_emulator_colors.clone(),
+                    self.terminal_emulator_color_codes.clone(),
                 );
                 self.tiled_panes
                     .split_pane_vertically(pid, Box::new(new_terminal), client_id);
@@ -844,8 +906,7 @@ impl Tab {
             || self
                 .suppressed_panes
                 .values()
-                .find(|s_p| s_p.pid() == PaneId::Terminal(pid))
-                .is_some()
+                .any(|s_p| s_p.pid() == PaneId::Terminal(pid))
     }
     pub fn handle_pty_bytes(&mut self, pid: RawFd, bytes: VteBytes) {
         if let Some(terminal_output) = self
@@ -906,6 +967,7 @@ impl Tab {
         });
     }
     pub fn write_to_active_terminal(&mut self, input_bytes: Vec<u8>, client_id: ClientId) {
+        self.clear_search(client_id); // this is an inexpensive operation if empty, if we need more such cleanups we should consider moving this and the rest to some sort of cleanup method
         let pane_id = if self.floating_panes.panes_are_visible() {
             self.floating_panes
                 .get_active_pane_id(client_id)
@@ -944,14 +1006,14 @@ impl Tab {
                         active_terminal_id,
                     ))
                     .unwrap();
-            }
+            },
             PaneId::Plugin(pid) => {
                 for key in parse_keys(&input_bytes) {
                     self.senders
                         .send_to_plugin(PluginInstruction::Update(Some(pid), None, Event::Key(key)))
                         .unwrap()
                 }
-            }
+            },
         }
     }
     pub fn get_active_terminal_cursor_position(
@@ -990,6 +1052,18 @@ impl Tab {
     pub fn are_floating_panes_visible(&self) -> bool {
         self.floating_panes.panes_are_visible()
     }
+    pub fn switch_next_pane_fullscreen(&mut self, client_id: ClientId) {
+        if !self.is_fullscreen_active() {
+            return;
+        }
+        self.tiled_panes.switch_next_pane_fullscreen(client_id);
+    }
+    pub fn switch_prev_pane_fullscreen(&mut self, client_id: ClientId) {
+        if !self.is_fullscreen_active() {
+            return;
+        }
+        self.tiled_panes.switch_next_pane_fullscreen(client_id);
+    }
     pub fn set_force_render(&mut self) {
         self.tiled_panes.set_force_render();
         self.floating_panes.set_force_render();
@@ -1087,11 +1161,11 @@ impl Tab {
                     ); // goto row/col
                     output.add_post_vte_instruction_to_client(client_id, show_cursor);
                     output.add_post_vte_instruction_to_client(client_id, goto_cursor_position);
-                }
+                },
                 None => {
                     let hide_cursor = "\u{1b}[?25l";
                     output.add_post_vte_instruction_to_client(client_id, hide_cursor);
-                }
+                },
             }
         }
     }
@@ -1144,7 +1218,7 @@ impl Tab {
                 .floating_panes
                 .resize_active_pane_left(client_id, &mut self.os_api);
             if successfully_resized {
-                self.set_force_render(); // we force render here to make sure the panes under the floating pane render and don't leave "garbage" incase of a decrease
+                self.set_force_render(); // we force render here to make sure the panes under the floating pane render and don't leave "garbage" in case of a decrease
             }
         } else {
             self.tiled_panes.resize_active_pane_left(client_id);
@@ -1156,7 +1230,7 @@ impl Tab {
                 .floating_panes
                 .resize_active_pane_right(client_id, &mut self.os_api);
             if successfully_resized {
-                self.set_force_render(); // we force render here to make sure the panes under the floating pane render and don't leave "garbage" incase of a decrease
+                self.set_force_render(); // we force render here to make sure the panes under the floating pane render and don't leave "garbage" in case of a decrease
             }
         } else {
             self.tiled_panes.resize_active_pane_right(client_id);
@@ -1168,7 +1242,7 @@ impl Tab {
                 .floating_panes
                 .resize_active_pane_down(client_id, &mut self.os_api);
             if successfully_resized {
-                self.set_force_render(); // we force render here to make sure the panes under the floating pane render and don't leave "garbage" incase of a decrease
+                self.set_force_render(); // we force render here to make sure the panes under the floating pane render and don't leave "garbage" in case of a decrease
             }
         } else {
             self.tiled_panes.resize_active_pane_down(client_id);
@@ -1180,7 +1254,7 @@ impl Tab {
                 .floating_panes
                 .resize_active_pane_up(client_id, &mut self.os_api);
             if successfully_resized {
-                self.set_force_render(); // we force render here to make sure the panes under the floating pane render and don't leave "garbage" incase of a decrease
+                self.set_force_render(); // we force render here to make sure the panes under the floating pane render and don't leave "garbage" in case of a decrease
             }
         } else {
             self.tiled_panes.resize_active_pane_up(client_id);
@@ -1192,7 +1266,7 @@ impl Tab {
                 .floating_panes
                 .resize_active_pane_increase(client_id, &mut self.os_api);
             if successfully_resized {
-                self.set_force_render(); // we force render here to make sure the panes under the floating pane render and don't leave "garbage" incase of a decrease
+                self.set_force_render(); // we force render here to make sure the panes under the floating pane render and don't leave "garbage" in case of a decrease
             }
         } else {
             self.tiled_panes.resize_active_pane_increase(client_id);
@@ -1204,7 +1278,7 @@ impl Tab {
                 .floating_panes
                 .resize_active_pane_decrease(client_id, &mut self.os_api);
             if successfully_resized {
-                self.set_force_render(); // we force render here to make sure the panes under the floating pane render and don't leave "garbage" incase of a decrease
+                self.set_force_render(); // we force render here to make sure the panes under the floating pane render and don't leave "garbage" in case of a decrease
             }
         } else {
             self.tiled_panes.resize_active_pane_decrease(client_id);
@@ -1222,6 +1296,7 @@ impl Tab {
             return;
         }
         if self.tiled_panes.fullscreen_is_active() {
+            self.switch_next_pane_fullscreen(client_id);
             return;
         }
         self.tiled_panes.focus_next_pane(client_id);
@@ -1231,6 +1306,7 @@ impl Tab {
             return;
         }
         if self.tiled_panes.fullscreen_is_active() {
+            self.switch_prev_pane_fullscreen(client_id);
             return;
         }
         self.tiled_panes.focus_previous_pane(client_id);
@@ -1247,7 +1323,8 @@ impl Tab {
                 return false;
             }
             if self.tiled_panes.fullscreen_is_active() {
-                return false;
+                self.switch_next_pane_fullscreen(client_id);
+                return true;
             }
             self.tiled_panes.move_focus_left(client_id)
         }
@@ -1296,7 +1373,8 @@ impl Tab {
                 return false;
             }
             if self.tiled_panes.fullscreen_is_active() {
-                return false;
+                self.switch_next_pane_fullscreen(client_id);
+                return true;
             }
             self.tiled_panes.move_focus_right(client_id)
         }
@@ -1498,7 +1576,6 @@ impl Tab {
         let mut file = temp_dir();
         file.push(format!("{}.dump", Uuid::new_v4()));
         self.dump_active_terminal_screen(Some(String::from(file.to_string_lossy())), client_id);
-        // let line_number = self.get_active_pane(client_id).map(|a_t| a_t.get_line_number());
         let line_number = self
             .get_active_pane(client_id)
             .and_then(|a_t| a_t.get_line_number());
@@ -1673,12 +1750,14 @@ impl Tab {
             let relative_position = pane.relative_position(position);
 
             if pane.mouse_mode() {
-                let mouse_event = format!(
-                    "\u{1b}[<0;{:?};{:?}M",
-                    relative_position.column.0 + 1,
-                    relative_position.line.0 + 1
-                );
-                self.write_to_active_terminal(mouse_event.into_bytes(), client_id);
+                if !pane.position_is_on_frame(position) {
+                    let mouse_event = format!(
+                        "\u{1b}[<0;{:?};{:?}M",
+                        relative_position.column() + 1,
+                        relative_position.line() + 1
+                    );
+                    self.write_to_active_terminal(mouse_event.into_bytes(), client_id);
+                }
             } else {
                 // TODO: rename this method, it is used to forward click events to plugin panes
                 pane.start_selection(&relative_position, client_id);
@@ -1694,12 +1773,14 @@ impl Tab {
         if let Some(pane) = self.get_pane_at(position, false) {
             let relative_position = pane.relative_position(position);
             if pane.mouse_mode() {
-                let mouse_event = format!(
-                    "\u{1b}[<2;{:?};{:?}M",
-                    relative_position.column.0 + 1,
-                    relative_position.line.0 + 1
-                );
-                self.write_to_active_terminal(mouse_event.into_bytes(), client_id);
+                if !pane.position_is_on_frame(position) {
+                    let mouse_event = format!(
+                        "\u{1b}[<2;{:?};{:?}M",
+                        relative_position.column() + 1,
+                        relative_position.line() + 1
+                    );
+                    self.write_to_active_terminal(mouse_event.into_bytes(), client_id);
+                }
             } else {
                 pane.handle_right_click(&relative_position, client_id);
             }
@@ -1741,11 +1822,11 @@ impl Tab {
             let relative_position = active_pane.relative_position(position);
             if active_pane.mouse_mode() {
                 // ensure that coordinates are valid
-                let col = (relative_position.column.0 + 1)
+                let col = (relative_position.column() + 1)
                     .max(1)
                     .min(active_pane.get_content_columns());
 
-                let line = (relative_position.line.0 + 1)
+                let line = (relative_position.line() + 1)
                     .max(1)
                     .min(active_pane.get_content_rows() as isize);
                 let mouse_event = format!("\u{1b}[<0;{:?};{:?}m", col, line);
@@ -1770,7 +1851,12 @@ impl Tab {
             }
         }
     }
-    pub fn handle_mouse_hold(&mut self, position_on_screen: &Position, client_id: ClientId) {
+    pub fn handle_mouse_hold(
+        &mut self,
+        position_on_screen: &Position,
+        client_id: ClientId,
+    ) -> bool {
+        // return value indicates whether we should trigger a render
         // determine if event is repeated to enable smooth scrolling
         let is_repeated = if let Some(last_position) = self.last_mouse_hold_position {
             position_on_screen == &last_position
@@ -1788,7 +1874,8 @@ impl Tab {
                 .move_pane_with_mouse(*position_on_screen, search_selectable)
         {
             self.set_force_render();
-            return;
+            return !is_repeated; // we don't need to re-render in this case if the pane did not move
+                                 // return;
         }
 
         let selecting = self.selecting_with_mouse;
@@ -1798,20 +1885,23 @@ impl Tab {
             let relative_position = active_pane.relative_position(position_on_screen);
             if active_pane.mouse_mode() && !is_repeated {
                 // ensure that coordinates are valid
-                let col = (relative_position.column.0 + 1)
+                let col = (relative_position.column() + 1)
                     .max(1)
                     .min(active_pane.get_content_columns());
 
-                let line = (relative_position.line.0 + 1)
+                let line = (relative_position.line() + 1)
                     .max(1)
                     .min(active_pane.get_content_rows() as isize);
 
                 let mouse_event = format!("\u{1b}[<32;{:?};{:?}M", col, line);
                 self.write_to_active_terminal(mouse_event.into_bytes(), client_id);
+                return true; // we need to re-render in this case so the selection disappears
             } else if selecting {
                 active_pane.update_selection(&relative_position, client_id);
+                return true; // we need to re-render in this case so the selection is updated
             }
         }
+        false // we shouldn't even get here, but might as well not needlessly render if we do
     }
 
     pub fn copy_selection(&self, client_id: ClientId) {
@@ -1847,11 +1937,11 @@ impl Tab {
                         .send_to_server(ServerInstruction::Render(Some(serialized_output)))
                         .unwrap();
                     Event::CopyToClipboard(self.clipboard_provider.as_copy_destination())
-                }
+                },
                 Err(err) => {
                     log::error!("could not write selection to clipboard: {}", err);
                     Event::SystemClipboardFailure
-                }
+                },
             };
         self.senders
             .send_to_plugin(PluginInstruction::Update(None, None, clipboard_event))
@@ -1917,6 +2007,21 @@ impl Tab {
         }
     }
 
+    pub fn undo_active_rename_pane(&mut self, client_id: ClientId) {
+        if let Some(active_terminal_id) = self.get_active_terminal_id(client_id) {
+            let active_terminal = if self.are_floating_panes_visible() {
+                self.floating_panes
+                    .get_pane_mut(PaneId::Terminal(active_terminal_id))
+            } else {
+                self.tiled_panes
+                    .get_pane_mut(PaneId::Terminal(active_terminal_id))
+            }
+            .unwrap();
+
+            active_terminal.load_pane_name();
+        }
+    }
+
     pub fn is_position_inside_viewport(&self, point: &Position) -> bool {
         let Position {
             line: Line(line),
@@ -1939,6 +2044,53 @@ impl Tab {
     pub fn panes_to_hide_count(&self) -> usize {
         self.tiled_panes.panes_to_hide_count()
     }
+
+    pub fn update_search_term(&mut self, buf: Vec<u8>, client_id: ClientId) {
+        if let Some(active_pane) = self.get_active_pane_or_floating_pane_mut(client_id) {
+            // It only allows printable unicode, delete and backspace keys.
+            let is_updatable = buf.iter().all(|u| matches!(u, 0x20..=0x7E | 0x08 | 0x7F));
+            if is_updatable {
+                let s = str::from_utf8(&buf).unwrap();
+                active_pane.update_search_term(s);
+            }
+        }
+    }
+
+    pub fn search_down(&mut self, client_id: ClientId) {
+        if let Some(active_pane) = self.get_active_pane_or_floating_pane_mut(client_id) {
+            active_pane.search_down();
+        }
+    }
+
+    pub fn search_up(&mut self, client_id: ClientId) {
+        if let Some(active_pane) = self.get_active_pane_or_floating_pane_mut(client_id) {
+            active_pane.search_up();
+        }
+    }
+
+    pub fn toggle_search_case_sensitivity(&mut self, client_id: ClientId) {
+        if let Some(active_pane) = self.get_active_pane_or_floating_pane_mut(client_id) {
+            active_pane.toggle_search_case_sensitivity();
+        }
+    }
+
+    pub fn toggle_search_wrap(&mut self, client_id: ClientId) {
+        if let Some(active_pane) = self.get_active_pane_or_floating_pane_mut(client_id) {
+            active_pane.toggle_search_wrap();
+        }
+    }
+
+    pub fn toggle_search_whole_words(&mut self, client_id: ClientId) {
+        if let Some(active_pane) = self.get_active_pane_or_floating_pane_mut(client_id) {
+            active_pane.toggle_search_whole_words();
+        }
+    }
+
+    pub fn clear_search(&mut self, client_id: ClientId) {
+        if let Some(active_pane) = self.get_active_pane_or_floating_pane_mut(client_id) {
+            active_pane.clear_search();
+        }
+    }
 }
 
 #[cfg(test)]
