@@ -1,6 +1,6 @@
 use super::{Output, Tab};
+use crate::panes::sixel::SixelImageStore;
 use crate::screen::CopyOptions;
-use crate::zellij_tile::data::{ModeInfo, Palette};
 use crate::Arc;
 use crate::Mutex;
 use crate::{
@@ -11,22 +11,21 @@ use crate::{
 };
 use std::convert::TryInto;
 use std::path::PathBuf;
-use zellij_tile::prelude::Style;
 use zellij_utils::envs::set_session_name;
 use zellij_utils::input::layout::LayoutTemplate;
 use zellij_utils::ipc::IpcReceiverWithContext;
-use zellij_utils::pane_size::Size;
+use zellij_utils::pane_size::{Size, SizeInPixels};
 use zellij_utils::position::Position;
 
 use std::cell::RefCell;
-use std::collections::HashMap;
-use std::collections::HashSet;
+use std::collections::{HashMap, HashSet};
 use std::os::unix::io::RawFd;
 use std::rc::Rc;
 
 use zellij_utils::nix;
 
 use zellij_utils::{
+    data::{InputMode, ModeInfo, Palette, Style},
     input::command::TerminalAction,
     interprocess::local_socket::LocalSocketStream,
     ipc::{ClientToServerMsg, ServerToClientMsg},
@@ -90,17 +89,73 @@ impl ServerOsApi for FakeInputOutput {
         unimplemented!()
     }
     fn write_to_file(&mut self, buf: String, name: Option<String>) {
-        let f: String;
-        match name {
-            Some(x) => f = x,
-            None => f = "tmp-name".to_owned(),
-        }
+        let f: String = match name {
+            Some(x) => x,
+            None => "tmp-name".to_owned(),
+        };
         self.file_dumps.lock().unwrap().insert(f, buf);
     }
 }
 
 // TODO: move to shared thingy with other test file
-fn create_new_tab(size: Size) -> Tab {
+fn create_new_tab(size: Size, default_mode: ModeInfo) -> Tab {
+    set_session_name("test".into());
+    let index = 0;
+    let position = 0;
+    let name = String::new();
+    let os_api = Box::new(FakeInputOutput {
+        file_dumps: Arc::new(Mutex::new(HashMap::new())),
+    });
+    let senders = ThreadSenders::default().silently_fail_on_send();
+    let max_panes = None;
+    let mode_info = default_mode;
+    let style = Style::default();
+    let draw_pane_frames = true;
+    let client_id = 1;
+    let session_is_mirrored = true;
+    let mut connected_clients = HashSet::new();
+    connected_clients.insert(client_id);
+    let connected_clients = Rc::new(RefCell::new(connected_clients));
+    let character_cell_info = Rc::new(RefCell::new(None));
+    let terminal_emulator_colors = Rc::new(RefCell::new(Palette::default()));
+    let copy_options = CopyOptions::default();
+    let terminal_emulator_color_codes = Rc::new(RefCell::new(HashMap::new()));
+    let sixel_image_store = Rc::new(RefCell::new(SixelImageStore::default()));
+    let mut tab = Tab::new(
+        index,
+        position,
+        name,
+        size,
+        character_cell_info,
+        sixel_image_store,
+        os_api,
+        senders,
+        max_panes,
+        style,
+        mode_info,
+        draw_pane_frames,
+        connected_clients,
+        session_is_mirrored,
+        client_id,
+        copy_options,
+        terminal_emulator_colors,
+        terminal_emulator_color_codes,
+    );
+    tab.apply_layout(
+        LayoutTemplate::default().try_into().unwrap(),
+        vec![1],
+        index,
+        client_id,
+    );
+    tab
+}
+
+fn create_new_tab_with_sixel_support(
+    size: Size,
+    sixel_image_store: Rc<RefCell<SixelImageStore>>,
+) -> Tab {
+    // this is like the create_new_tab function but includes stuff needed for sixel,
+    // eg. character_cell_size
     set_session_name("test".into());
     let index = 0;
     let position = 0;
@@ -118,15 +173,20 @@ fn create_new_tab(size: Size) -> Tab {
     let mut connected_clients = HashSet::new();
     connected_clients.insert(client_id);
     let connected_clients = Rc::new(RefCell::new(connected_clients));
-    let character_cell_info = Rc::new(RefCell::new(None));
+    let character_cell_size = Rc::new(RefCell::new(Some(SizeInPixels {
+        width: 8,
+        height: 21,
+    })));
     let terminal_emulator_colors = Rc::new(RefCell::new(Palette::default()));
     let copy_options = CopyOptions::default();
+    let terminal_emulator_color_codes = Rc::new(RefCell::new(HashMap::new()));
     let mut tab = Tab::new(
         index,
         position,
         name,
         size,
-        character_cell_info,
+        character_cell_size,
+        sixel_image_store,
         os_api,
         senders,
         max_panes,
@@ -138,6 +198,7 @@ fn create_new_tab(size: Size) -> Tab {
         client_id,
         copy_options,
         terminal_emulator_colors,
+        terminal_emulator_color_codes,
     );
     tab.apply_layout(
         LayoutTemplate::default().try_into().unwrap(),
@@ -160,16 +221,52 @@ fn read_fixture(fixture_name: &str) -> Vec<u8> {
 
 use crate::panes::grid::Grid;
 use crate::panes::link_handler::LinkHandler;
-use ::insta::assert_snapshot;
+use insta::assert_snapshot;
 use zellij_utils::vte;
 
 fn take_snapshot(ansi_instructions: &str, rows: usize, columns: usize, palette: Palette) -> String {
+    let sixel_image_store = Rc::new(RefCell::new(SixelImageStore::default()));
+    let terminal_emulator_color_codes = Rc::new(RefCell::new(HashMap::new()));
+    let character_cell_size = Rc::new(RefCell::new(Some(SizeInPixels {
+        width: 8,
+        height: 21,
+    })));
     let mut grid = Grid::new(
         rows,
         columns,
         Rc::new(RefCell::new(palette)),
+        terminal_emulator_color_codes,
         Rc::new(RefCell::new(LinkHandler::new())),
-        Rc::new(RefCell::new(None)),
+        character_cell_size,
+        sixel_image_store,
+    );
+    let mut vte_parser = vte::Parser::new();
+    for &byte in ansi_instructions.as_bytes() {
+        vte_parser.advance(&mut grid, byte);
+    }
+    format!("{:?}", grid)
+}
+
+fn take_snapshot_with_sixel(
+    ansi_instructions: &str,
+    rows: usize,
+    columns: usize,
+    palette: Palette,
+    sixel_image_store: Rc<RefCell<SixelImageStore>>,
+) -> String {
+    let terminal_emulator_color_codes = Rc::new(RefCell::new(HashMap::new()));
+    let character_cell_size = Rc::new(RefCell::new(Some(SizeInPixels {
+        width: 8,
+        height: 21,
+    })));
+    let mut grid = Grid::new(
+        rows,
+        columns,
+        Rc::new(RefCell::new(palette)),
+        terminal_emulator_color_codes,
+        Rc::new(RefCell::new(LinkHandler::new())),
+        character_cell_size,
+        sixel_image_store,
     );
     let mut vte_parser = vte::Parser::new();
     for &byte in ansi_instructions.as_bytes() {
@@ -185,12 +282,16 @@ fn take_snapshot_and_cursor_position(
     palette: Palette,
 ) -> (String, Option<(usize, usize)>) {
     // snapshot, x_coordinates, y_coordinates
+    let sixel_image_store = Rc::new(RefCell::new(SixelImageStore::default()));
+    let terminal_emulator_color_codes = Rc::new(RefCell::new(HashMap::new()));
     let mut grid = Grid::new(
         rows,
         columns,
         Rc::new(RefCell::new(palette)),
+        terminal_emulator_color_codes,
         Rc::new(RefCell::new(LinkHandler::new())),
         Rc::new(RefCell::new(None)),
+        sixel_image_store,
     );
     let mut vte_parser = vte::Parser::new();
     for &byte in ansi_instructions.as_bytes() {
@@ -206,7 +307,7 @@ fn dump_screen() {
         rows: 20,
     };
     let client_id = 1;
-    let mut tab = create_new_tab(size);
+    let mut tab = create_new_tab(size, ModeInfo::default());
     let map = Arc::new(Mutex::new(HashMap::new()));
     tab.os_api = Box::new(FakeInputOutput {
         file_dumps: map.clone(),
@@ -230,7 +331,7 @@ fn new_floating_pane() {
         rows: 20,
     };
     let client_id = 1;
-    let mut tab = create_new_tab(size);
+    let mut tab = create_new_tab(size, ModeInfo::default());
     let new_pane_id = PaneId::Terminal(2);
     let mut output = Output::default();
     tab.toggle_floating_panes(client_id, None);
@@ -256,7 +357,7 @@ fn floating_panes_persist_across_toggles() {
         rows: 20,
     };
     let client_id = 1;
-    let mut tab = create_new_tab(size);
+    let mut tab = create_new_tab(size, ModeInfo::default());
     let new_pane_id = PaneId::Terminal(2);
     let mut output = Output::default();
     tab.toggle_floating_panes(client_id, None);
@@ -286,7 +387,7 @@ fn toggle_floating_panes_off() {
         rows: 20,
     };
     let client_id = 1;
-    let mut tab = create_new_tab(size);
+    let mut tab = create_new_tab(size, ModeInfo::default());
     let new_pane_id = PaneId::Terminal(2);
     let mut output = Output::default();
     tab.toggle_floating_panes(client_id, None);
@@ -313,7 +414,7 @@ fn toggle_floating_panes_on() {
         rows: 20,
     };
     let client_id = 1;
-    let mut tab = create_new_tab(size);
+    let mut tab = create_new_tab(size, ModeInfo::default());
     let new_pane_id = PaneId::Terminal(2);
     let mut output = Output::default();
     tab.toggle_floating_panes(client_id, None);
@@ -341,7 +442,7 @@ fn five_new_floating_panes() {
         rows: 20,
     };
     let client_id = 1;
-    let mut tab = create_new_tab(size);
+    let mut tab = create_new_tab(size, ModeInfo::default());
     let new_pane_id_1 = PaneId::Terminal(2);
     let new_pane_id_2 = PaneId::Terminal(3);
     let new_pane_id_3 = PaneId::Terminal(4);
@@ -379,7 +480,7 @@ fn increase_floating_pane_size() {
         rows: 20,
     };
     let client_id = 1;
-    let mut tab = create_new_tab(size);
+    let mut tab = create_new_tab(size, ModeInfo::default());
     let new_pane_id_1 = PaneId::Terminal(2);
     let mut output = Output::default();
     tab.toggle_floating_panes(client_id, None);
@@ -406,7 +507,7 @@ fn decrease_floating_pane_size() {
         rows: 20,
     };
     let client_id = 1;
-    let mut tab = create_new_tab(size);
+    let mut tab = create_new_tab(size, ModeInfo::default());
     let new_pane_id_1 = PaneId::Terminal(2);
     let mut output = Output::default();
     tab.toggle_floating_panes(client_id, None);
@@ -433,7 +534,7 @@ fn resize_floating_pane_left() {
         rows: 20,
     };
     let client_id = 1;
-    let mut tab = create_new_tab(size);
+    let mut tab = create_new_tab(size, ModeInfo::default());
     let new_pane_id_1 = PaneId::Terminal(2);
     let mut output = Output::default();
     tab.toggle_floating_panes(client_id, None);
@@ -460,7 +561,7 @@ fn resize_floating_pane_right() {
         rows: 20,
     };
     let client_id = 1;
-    let mut tab = create_new_tab(size);
+    let mut tab = create_new_tab(size, ModeInfo::default());
     let new_pane_id_1 = PaneId::Terminal(2);
     let mut output = Output::default();
     tab.toggle_floating_panes(client_id, None);
@@ -487,7 +588,7 @@ fn resize_floating_pane_up() {
         rows: 20,
     };
     let client_id = 1;
-    let mut tab = create_new_tab(size);
+    let mut tab = create_new_tab(size, ModeInfo::default());
     let new_pane_id_1 = PaneId::Terminal(2);
     let mut output = Output::default();
     tab.toggle_floating_panes(client_id, None);
@@ -514,7 +615,7 @@ fn resize_floating_pane_down() {
         rows: 20,
     };
     let client_id = 1;
-    let mut tab = create_new_tab(size);
+    let mut tab = create_new_tab(size, ModeInfo::default());
     let new_pane_id_1 = PaneId::Terminal(2);
     let mut output = Output::default();
     tab.toggle_floating_panes(client_id, None);
@@ -541,7 +642,7 @@ fn move_floating_pane_focus_left() {
         rows: 20,
     };
     let client_id = 1;
-    let mut tab = create_new_tab(size);
+    let mut tab = create_new_tab(size, ModeInfo::default());
     let new_pane_id_1 = PaneId::Terminal(2);
     let new_pane_id_2 = PaneId::Terminal(3);
     let new_pane_id_3 = PaneId::Terminal(4);
@@ -586,7 +687,7 @@ fn move_floating_pane_focus_right() {
         rows: 20,
     };
     let client_id = 1;
-    let mut tab = create_new_tab(size);
+    let mut tab = create_new_tab(size, ModeInfo::default());
     let new_pane_id_1 = PaneId::Terminal(2);
     let new_pane_id_2 = PaneId::Terminal(3);
     let new_pane_id_3 = PaneId::Terminal(4);
@@ -632,7 +733,7 @@ fn move_floating_pane_focus_up() {
         rows: 20,
     };
     let client_id = 1;
-    let mut tab = create_new_tab(size);
+    let mut tab = create_new_tab(size, ModeInfo::default());
     let new_pane_id_1 = PaneId::Terminal(2);
     let new_pane_id_2 = PaneId::Terminal(3);
     let new_pane_id_3 = PaneId::Terminal(4);
@@ -677,7 +778,7 @@ fn move_floating_pane_focus_down() {
         rows: 20,
     };
     let client_id = 1;
-    let mut tab = create_new_tab(size);
+    let mut tab = create_new_tab(size, ModeInfo::default());
     let new_pane_id_1 = PaneId::Terminal(2);
     let new_pane_id_2 = PaneId::Terminal(3);
     let new_pane_id_3 = PaneId::Terminal(4);
@@ -723,7 +824,7 @@ fn move_floating_pane_focus_with_mouse() {
         rows: 20,
     };
     let client_id = 1;
-    let mut tab = create_new_tab(size);
+    let mut tab = create_new_tab(size, ModeInfo::default());
     let new_pane_id_1 = PaneId::Terminal(2);
     let new_pane_id_2 = PaneId::Terminal(3);
     let new_pane_id_3 = PaneId::Terminal(4);
@@ -769,7 +870,7 @@ fn move_pane_focus_with_mouse_to_non_floating_pane() {
         rows: 20,
     };
     let client_id = 1;
-    let mut tab = create_new_tab(size);
+    let mut tab = create_new_tab(size, ModeInfo::default());
     let new_pane_id_1 = PaneId::Terminal(2);
     let new_pane_id_2 = PaneId::Terminal(3);
     let new_pane_id_3 = PaneId::Terminal(4);
@@ -815,7 +916,7 @@ fn drag_pane_with_mouse() {
         rows: 20,
     };
     let client_id = 1;
-    let mut tab = create_new_tab(size);
+    let mut tab = create_new_tab(size, ModeInfo::default());
     let new_pane_id_1 = PaneId::Terminal(2);
     let new_pane_id_2 = PaneId::Terminal(3);
     let new_pane_id_3 = PaneId::Terminal(4);
@@ -861,7 +962,7 @@ fn mark_text_inside_floating_pane() {
         rows: 20,
     };
     let client_id = 1;
-    let mut tab = create_new_tab(size);
+    let mut tab = create_new_tab(size, ModeInfo::default());
     let new_pane_id_1 = PaneId::Terminal(2);
     let new_pane_id_2 = PaneId::Terminal(3);
     let new_pane_id_3 = PaneId::Terminal(4);
@@ -915,7 +1016,7 @@ fn resize_tab_with_floating_panes() {
         rows: 20,
     };
     let client_id = 1;
-    let mut tab = create_new_tab(size);
+    let mut tab = create_new_tab(size, ModeInfo::default());
     let new_pane_id_1 = PaneId::Terminal(2);
     let new_pane_id_2 = PaneId::Terminal(3);
     let new_pane_id_3 = PaneId::Terminal(4);
@@ -958,7 +1059,7 @@ fn shrink_whole_tab_with_floating_panes_horizontally_and_vertically() {
         rows: 20,
     };
     let client_id = 1;
-    let mut tab = create_new_tab(size);
+    let mut tab = create_new_tab(size, ModeInfo::default());
     let new_pane_id_1 = PaneId::Terminal(2);
     let new_pane_id_2 = PaneId::Terminal(3);
     let new_pane_id_3 = PaneId::Terminal(4);
@@ -998,7 +1099,7 @@ fn shrink_whole_tab_with_floating_panes_horizontally_and_vertically_and_expand_b
         rows: 20,
     };
     let client_id = 1;
-    let mut tab = create_new_tab(size);
+    let mut tab = create_new_tab(size, ModeInfo::default());
     let new_pane_id_1 = PaneId::Terminal(2);
     let new_pane_id_2 = PaneId::Terminal(3);
     let new_pane_id_3 = PaneId::Terminal(4);
@@ -1042,7 +1143,7 @@ fn embed_floating_pane() {
         rows: 20,
     };
     let client_id = 1;
-    let mut tab = create_new_tab(size);
+    let mut tab = create_new_tab(size, ModeInfo::default());
     let new_pane_id = PaneId::Terminal(2);
     let mut output = Output::default();
     tab.toggle_floating_panes(client_id, None);
@@ -1069,7 +1170,7 @@ fn float_embedded_pane() {
         rows: 20,
     };
     let client_id = 1;
-    let mut tab = create_new_tab(size);
+    let mut tab = create_new_tab(size, ModeInfo::default());
     let new_pane_id = PaneId::Terminal(2);
     let mut output = Output::default();
     tab.new_pane(new_pane_id, Some(client_id));
@@ -1095,7 +1196,7 @@ fn cannot_float_only_embedded_pane() {
         rows: 20,
     };
     let client_id = 1;
-    let mut tab = create_new_tab(size);
+    let mut tab = create_new_tab(size, ModeInfo::default());
     let mut output = Output::default();
     tab.handle_pty_bytes(
         1,
@@ -1122,7 +1223,7 @@ fn replacing_existing_wide_characters() {
         rows: 48,
     };
     let client_id = 1;
-    let mut tab = create_new_tab(size);
+    let mut tab = create_new_tab(size, ModeInfo::default());
     let mut output = Output::default();
     let pane_content = read_fixture("ncmpcpp-wide-chars");
     tab.handle_pty_bytes(1, pane_content);
@@ -1143,7 +1244,7 @@ fn rename_embedded_pane() {
         rows: 20,
     };
     let client_id = 1;
-    let mut tab = create_new_tab(size);
+    let mut tab = create_new_tab(size, ModeInfo::default());
     let mut output = Output::default();
     tab.handle_pty_bytes(
         1,
@@ -1167,7 +1268,7 @@ fn rename_floating_pane() {
         rows: 20,
     };
     let client_id = 1;
-    let mut tab = create_new_tab(size);
+    let mut tab = create_new_tab(size, ModeInfo::default());
     let new_pane_id = PaneId::Terminal(2);
     let mut output = Output::default();
     tab.new_pane(new_pane_id, Some(client_id));
@@ -1195,7 +1296,7 @@ fn wide_characters_in_left_title_side() {
         rows: 48,
     };
     let client_id = 1;
-    let mut tab = create_new_tab(size);
+    let mut tab = create_new_tab(size, ModeInfo::default());
     let mut output = Output::default();
     let pane_content = read_fixture("title-wide-chars");
     tab.handle_pty_bytes(1, pane_content);
@@ -1215,7 +1316,7 @@ fn save_cursor_position_across_resizes() {
     // resize the pane
     let size = Size { cols: 100, rows: 5 };
     let client_id = 1;
-    let mut tab = create_new_tab(size);
+    let mut tab = create_new_tab(size, ModeInfo::default());
     let mut output = Output::default();
 
     tab.handle_pty_bytes(
@@ -1235,6 +1336,76 @@ fn save_cursor_position_across_resizes() {
     assert_snapshot!(snapshot);
 }
 
+#[test]
+fn move_floating_pane_with_sixel_image() {
+    let new_pane_id = PaneId::Terminal(2);
+    let size = Size {
+        cols: 121,
+        rows: 20,
+    };
+    let client_id = 1;
+    let sixel_image_store = Rc::new(RefCell::new(SixelImageStore::default()));
+    let mut tab = create_new_tab_with_sixel_support(size, sixel_image_store.clone());
+    let character_cell_size = Rc::new(RefCell::new(Some(SizeInPixels {
+        width: 8,
+        height: 21,
+    })));
+    let mut output = Output::new(sixel_image_store.clone(), character_cell_size);
+
+    tab.toggle_floating_panes(client_id, None);
+    tab.new_pane(new_pane_id, Some(client_id));
+    let fixture = read_fixture("sixel-image-500px.six");
+    tab.handle_pty_bytes(2, fixture);
+    tab.handle_left_click(&Position::new(5, 71), client_id);
+    tab.handle_mouse_release(&Position::new(7, 75), client_id);
+
+    tab.render(&mut output, None);
+    let snapshot = take_snapshot_with_sixel(
+        output.serialize().get(&client_id).unwrap(),
+        size.rows,
+        size.cols,
+        Palette::default(),
+        sixel_image_store,
+    );
+
+    assert_snapshot!(snapshot);
+}
+
+#[test]
+fn floating_pane_above_sixel_image() {
+    let new_pane_id = PaneId::Terminal(2);
+    let size = Size {
+        cols: 121,
+        rows: 20,
+    };
+    let client_id = 1;
+    let sixel_image_store = Rc::new(RefCell::new(SixelImageStore::default()));
+    let mut tab = create_new_tab_with_sixel_support(size, sixel_image_store.clone());
+    let character_cell_size = Rc::new(RefCell::new(Some(SizeInPixels {
+        width: 8,
+        height: 21,
+    })));
+    let mut output = Output::new(sixel_image_store.clone(), character_cell_size);
+
+    tab.toggle_floating_panes(client_id, None);
+    tab.new_pane(new_pane_id, Some(client_id));
+    let fixture = read_fixture("sixel-image-500px.six");
+    tab.handle_pty_bytes(1, fixture);
+    tab.handle_left_click(&Position::new(5, 71), client_id);
+    tab.handle_mouse_release(&Position::new(7, 75), client_id);
+
+    tab.render(&mut output, None);
+    let snapshot = take_snapshot_with_sixel(
+        output.serialize().get(&client_id).unwrap(),
+        size.rows,
+        size.cols,
+        Palette::default(),
+        sixel_image_store,
+    );
+
+    assert_snapshot!(snapshot);
+}
+
 #[test]
 fn suppress_tiled_pane() {
     let size = Size {
@@ -1242,7 +1413,7 @@ fn suppress_tiled_pane() {
         rows: 20,
     };
     let client_id = 1;
-    let mut tab = create_new_tab(size);
+    let mut tab = create_new_tab(size, ModeInfo::default());
     let new_pane_id = PaneId::Terminal(2);
     let mut output = Output::default();
     tab.suppress_active_pane(new_pane_id, client_id);
@@ -1264,7 +1435,7 @@ fn suppress_floating_pane() {
         rows: 20,
     };
     let client_id = 1;
-    let mut tab = create_new_tab(size);
+    let mut tab = create_new_tab(size, ModeInfo::default());
     let new_pane_id = PaneId::Terminal(2);
     let editor_pane_id = PaneId::Terminal(3);
     let mut output = Output::default();
@@ -1290,7 +1461,7 @@ fn close_suppressing_tiled_pane() {
         rows: 20,
     };
     let client_id = 1;
-    let mut tab = create_new_tab(size);
+    let mut tab = create_new_tab(size, ModeInfo::default());
     let new_pane_id = PaneId::Terminal(2);
     let mut output = Output::default();
     tab.suppress_active_pane(new_pane_id, client_id);
@@ -1314,7 +1485,7 @@ fn close_suppressing_floating_pane() {
         rows: 20,
     };
     let client_id = 1;
-    let mut tab = create_new_tab(size);
+    let mut tab = create_new_tab(size, ModeInfo::default());
     let new_pane_id = PaneId::Terminal(2);
     let editor_pane_id = PaneId::Terminal(3);
     let mut output = Output::default();
@@ -1342,7 +1513,7 @@ fn suppress_tiled_pane_float_it_and_close() {
         rows: 20,
     };
     let client_id = 1;
-    let mut tab = create_new_tab(size);
+    let mut tab = create_new_tab(size, ModeInfo::default());
     let new_pane_id = PaneId::Terminal(2);
     let mut output = Output::default();
     tab.suppress_active_pane(new_pane_id, client_id);
@@ -1367,7 +1538,7 @@ fn suppress_floating_pane_embed_it_and_close_it() {
         rows: 20,
     };
     let client_id = 1;
-    let mut tab = create_new_tab(size);
+    let mut tab = create_new_tab(size, ModeInfo::default());
     let new_pane_id = PaneId::Terminal(2);
     let editor_pane_id = PaneId::Terminal(3);
     let mut output = Output::default();
@@ -1396,7 +1567,7 @@ fn resize_whole_tab_while_tiled_pane_is_suppressed() {
         rows: 20,
     };
     let client_id = 1;
-    let mut tab = create_new_tab(size);
+    let mut tab = create_new_tab(size, ModeInfo::default());
     let new_pane_id = PaneId::Terminal(2);
     let mut output = Output::default();
     tab.suppress_active_pane(new_pane_id, client_id);
@@ -1422,7 +1593,7 @@ fn resize_whole_tab_while_floting_pane_is_suppressed() {
         rows: 20,
     };
     let client_id = 1;
-    let mut tab = create_new_tab(size);
+    let mut tab = create_new_tab(size, ModeInfo::default());
     let new_pane_id = PaneId::Terminal(2);
     let editor_pane_id = PaneId::Terminal(3);
     let mut output = Output::default();
@@ -1444,3 +1615,108 @@ fn resize_whole_tab_while_floting_pane_is_suppressed() {
     );
     assert_snapshot!(snapshot);
 }
+
+#[test]
+fn enter_search_pane() {
+    let size = Size {
+        cols: 121,
+        rows: 20,
+    };
+    let client_id = 1;
+    let mode_info = ModeInfo {
+        mode: InputMode::Search,
+        ..Default::default()
+    };
+    let mut tab = create_new_tab(size, mode_info);
+    let mut output = Output::default();
+    let pane_content = read_fixture("grid_copy");
+    tab.handle_pty_bytes(1, pane_content);
+    tab.render(&mut output, None);
+    let snapshot = take_snapshot(
+        output.serialize().get(&client_id).unwrap(),
+        size.rows,
+        size.cols,
+        Palette::default(),
+    );
+    assert_snapshot!("search_tab_nothing_highlighted", snapshot);
+
+    // Pane title should show 'tortor' as search term
+    // Only lines containing 'tortor' get marked as render-targets, so
+    // only those are updated (search-styling is not visible here).
+    tab.update_search_term("tortor".as_bytes().to_vec(), client_id);
+    tab.render(&mut output, None);
+    let snapshot = take_snapshot(
+        output.serialize().get(&client_id).unwrap(),
+        size.rows,
+        size.cols,
+        Palette::default(),
+    );
+    assert_snapshot!("search_tab_highlight_tortor", snapshot);
+
+    // Pane title should show search modifiers
+    tab.toggle_search_wrap(client_id);
+    tab.toggle_search_whole_words(client_id);
+    tab.toggle_search_case_sensitivity(client_id);
+    tab.render(&mut output, None);
+    let snapshot = take_snapshot(
+        output.serialize().get(&client_id).unwrap(),
+        size.rows,
+        size.cols,
+        Palette::default(),
+    );
+    assert_snapshot!("search_tab_highlight_tortor_modified", snapshot);
+
+    // And only the search term again
+    tab.toggle_search_wrap(client_id);
+    tab.toggle_search_whole_words(client_id);
+    tab.toggle_search_case_sensitivity(client_id);
+
+    tab.render(&mut output, None);
+    let snapshot = take_snapshot(
+        output.serialize().get(&client_id).unwrap(),
+        size.rows,
+        size.cols,
+        Palette::default(),
+    );
+    assert_snapshot!("search_tab_highlight_tortor", snapshot);
+}
+
+#[test]
+fn enter_search_floating_pane() {
+    let size = Size {
+        cols: 121,
+        rows: 20,
+    };
+    let client_id = 1;
+    let mode_info = ModeInfo {
+        mode: InputMode::Search,
+        ..Default::default()
+    };
+    let mut tab = create_new_tab(size, mode_info);
+    let new_pane_id = PaneId::Terminal(2);
+    let mut output = Output::default();
+    tab.toggle_floating_panes(client_id, None);
+    tab.new_pane(new_pane_id, Some(client_id));
+
+    let pane_content = read_fixture("grid_copy");
+    tab.handle_pty_bytes(2, pane_content);
+    tab.render(&mut output, None);
+    let snapshot = take_snapshot(
+        output.serialize().get(&client_id).unwrap(),
+        size.rows,
+        size.cols,
+        Palette::default(),
+    );
+    assert_snapshot!("search_floating_tab_nothing_highlighted", snapshot);
+
+    // Only the line inside the floating tab which contain 'fring' should be in the new snapshot
+    tab.update_search_term("fring".as_bytes().to_vec(), client_id);
+    tab.render(&mut output, None);
+    let snapshot = take_snapshot(
+        output.serialize().get(&client_id).unwrap(),
+        size.rows,
+        size.cols,
+        Palette::default(),
+    );
+    assert_snapshot!("search_floating_tab_highlight_fring", snapshot);
+}
