@@ -1,19 +1,24 @@
-use zellij_utils::{pane_size::Viewport, zellij_tile};
+use zellij_utils::pane_size::Viewport;
 
+use crate::output::CharacterChunk;
+use crate::panes::terminal_character::{TerminalCharacter, EMPTY_TERMINAL_CHARACTER, RESET_STYLES};
 use crate::tab::Pane;
 use ansi_term::Colour::{Fixed, RGB};
 use std::collections::HashMap;
-use zellij_tile::data::PaletteColor;
-use zellij_utils::shared::colors;
+use zellij_utils::{data::PaletteColor, shared::colors};
 
 use std::fmt::{Display, Error, Formatter};
 pub mod boundary_type {
     pub const TOP_RIGHT: &str = "┐";
+    pub const TOP_RIGHT_ROUND: &str = "╮";
     pub const VERTICAL: &str = "│";
     pub const HORIZONTAL: &str = "─";
     pub const TOP_LEFT: &str = "┌";
+    pub const TOP_LEFT_ROUND: &str = "╭";
     pub const BOTTOM_RIGHT: &str = "┘";
+    pub const BOTTOM_RIGHT_ROUND: &str = "╯";
     pub const BOTTOM_LEFT: &str = "└";
+    pub const BOTTOM_LEFT_ROUND: &str = "╰";
     pub const VERTICAL_LEFT: &str = "┤";
     pub const VERTICAL_RIGHT: &str = "├";
     pub const HORIZONTAL_DOWN: &str = "┬";
@@ -42,6 +47,19 @@ impl BoundarySymbol {
         self.color = color;
         *self
     }
+    pub fn as_terminal_character(&self) -> TerminalCharacter {
+        if self.invisible {
+            EMPTY_TERMINAL_CHARACTER
+        } else {
+            let character = self.boundary_type.chars().next().unwrap();
+            TerminalCharacter {
+                character,
+                width: 1,
+                styles: RESET_STYLES
+                    .foreground(self.color.map(|palette_color| palette_color.into())),
+            }
+        }
+    }
 }
 
 impl Display for BoundarySymbol {
@@ -52,10 +70,10 @@ impl Display for BoundarySymbol {
                 Some(color) => match color {
                     PaletteColor::Rgb((r, g, b)) => {
                         write!(f, "{}", RGB(r, g, b).paint(self.boundary_type))
-                    }
+                    },
                     PaletteColor::EightBit(color) => {
                         write!(f, "{}", Fixed(color).paint(self.boundary_type))
-                    }
+                    },
                 },
                 None => write!(f, "{}", self.boundary_type),
             },
@@ -79,7 +97,7 @@ fn combine_symbols(
                 invisible,
                 color,
             })
-        }
+        },
         (TOP_RIGHT, TOP_RIGHT) => {
             // (┐, ┐) => Some(┐)
             let boundary_type = TOP_RIGHT;
@@ -88,7 +106,7 @@ fn combine_symbols(
                 invisible,
                 color,
             })
-        }
+        },
         (TOP_RIGHT, VERTICAL) | (TOP_RIGHT, BOTTOM_RIGHT) | (TOP_RIGHT, VERTICAL_LEFT) => {
             // (┐, │) => Some(┤)
             // (┐, ┘) => Some(┤)
@@ -99,7 +117,7 @@ fn combine_symbols(
                 invisible,
                 color,
             })
-        }
+        },
         (TOP_RIGHT, HORIZONTAL) | (TOP_RIGHT, TOP_LEFT) | (TOP_RIGHT, HORIZONTAL_DOWN) => {
             // (┐, ─) => Some(┬)
             // (┐, ┌) => Some(┬)
@@ -110,7 +128,7 @@ fn combine_symbols(
                 invisible,
                 color,
             })
-        }
+        },
         (TOP_RIGHT, BOTTOM_LEFT) | (TOP_RIGHT, VERTICAL_RIGHT) | (TOP_RIGHT, HORIZONTAL_UP) => {
             // (┐, └) => Some(┼)
             // (┐, ├) => Some(┼)
@@ -121,7 +139,7 @@ fn combine_symbols(
                 invisible,
                 color,
             })
-        }
+        },
         (HORIZONTAL, HORIZONTAL) => {
             // (─, ─) => Some(─)
             let boundary_type = HORIZONTAL;
@@ -130,7 +148,7 @@ fn combine_symbols(
                 invisible,
                 color,
             })
-        }
+        },
         (HORIZONTAL, VERTICAL) | (HORIZONTAL, VERTICAL_LEFT) | (HORIZONTAL, VERTICAL_RIGHT) => {
             // (─, │) => Some(┼)
             // (─, ┤) => Some(┼)
@@ -141,7 +159,7 @@ fn combine_symbols(
                 invisible,
                 color,
             })
-        }
+        },
         (HORIZONTAL, TOP_LEFT) | (HORIZONTAL, HORIZONTAL_DOWN) => {
             // (─, ┌) => Some(┬)
             // (─, ┬) => Some(┬)
@@ -151,7 +169,7 @@ fn combine_symbols(
                 invisible,
                 color,
             })
-        }
+        },
         (HORIZONTAL, BOTTOM_RIGHT) | (HORIZONTAL, BOTTOM_LEFT) | (HORIZONTAL, HORIZONTAL_UP) => {
             // (─, ┘) => Some(┴)
             // (─, └) => Some(┴)
@@ -162,7 +180,7 @@ fn combine_symbols(
                 invisible,
                 color,
             })
-        }
+        },
         (VERTICAL, VERTICAL) => {
             // (│, │) => Some(│)
             let boundary_type = VERTICAL;
@@ -171,7 +189,7 @@ fn combine_symbols(
                 invisible,
                 color,
             })
-        }
+        },
         (VERTICAL, TOP_LEFT) | (VERTICAL, BOTTOM_LEFT) | (VERTICAL, VERTICAL_RIGHT) => {
             // (│, ┌) => Some(├)
             // (│, └) => Some(├)
@@ -182,7 +200,7 @@ fn combine_symbols(
                 invisible,
                 color,
             })
-        }
+        },
         (VERTICAL, BOTTOM_RIGHT) | (VERTICAL, VERTICAL_LEFT) => {
             // (│, ┘) => Some(┤)
             // (│, ┤) => Some(┤)
@@ -192,7 +210,7 @@ fn combine_symbols(
                 invisible,
                 color,
             })
-        }
+        },
         (VERTICAL, HORIZONTAL_DOWN) | (VERTICAL, HORIZONTAL_UP) => {
             // (│, ┬) => Some(┼)
             // (│, ┴) => Some(┼)
@@ -202,7 +220,7 @@ fn combine_symbols(
                 invisible,
                 color,
             })
-        }
+        },
         (TOP_LEFT, TOP_LEFT) => {
             // (┌, ┌) => Some(┌)
             let boundary_type = TOP_LEFT;
@@ -211,7 +229,7 @@ fn combine_symbols(
                 invisible,
                 color,
             })
-        }
+        },
         (TOP_LEFT, BOTTOM_RIGHT) | (TOP_LEFT, VERTICAL_LEFT) | (TOP_LEFT, HORIZONTAL_UP) => {
             // (┌, ┘) => Some(┼)
             // (┌, ┤) => Some(┼)
@@ -222,7 +240,7 @@ fn combine_symbols(
                 invisible,
                 color,
             })
-        }
+        },
         (TOP_LEFT, BOTTOM_LEFT) | (TOP_LEFT, VERTICAL_RIGHT) => {
             // (┌, └) => Some(├)
             // (┌, ├) => Some(├)
@@ -232,7 +250,7 @@ fn combine_symbols(
                 invisible,
                 color,
             })
-        }
+        },
         (TOP_LEFT, HORIZONTAL_DOWN) => {
             // (┌, ┬) => Some(┬)
             let boundary_type = HORIZONTAL_DOWN;
@@ -241,7 +259,7 @@ fn combine_symbols(
                 invisible,
                 color,
             })
-        }
+        },
         (BOTTOM_RIGHT, BOTTOM_RIGHT) => {
             // (┘, ┘) => Some(┘)
             let boundary_type = BOTTOM_RIGHT;
@@ -250,7 +268,7 @@ fn combine_symbols(
                 invisible,
                 color,
             })
-        }
+        },
         (BOTTOM_RIGHT, BOTTOM_LEFT) | (BOTTOM_RIGHT, HORIZONTAL_UP) => {
             // (┘, └) => Some(┴)
             // (┘, ┴) => Some(┴)
@@ -260,7 +278,7 @@ fn combine_symbols(
                 invisible,
                 color,
             })
-        }
+        },
         (BOTTOM_RIGHT, VERTICAL_LEFT) => {
             // (┘, ┤) => Some(┤)
             let boundary_type = VERTICAL_LEFT;
@@ -269,7 +287,7 @@ fn combine_symbols(
                 invisible,
                 color,
             })
-        }
+        },
         (BOTTOM_RIGHT, VERTICAL_RIGHT) | (BOTTOM_RIGHT, HORIZONTAL_DOWN) => {
             // (┘, ├) => Some(┼)
             // (┘, ┬) => Some(┼)
@@ -279,7 +297,7 @@ fn combine_symbols(
                 invisible,
                 color,
             })
-        }
+        },
         (BOTTOM_LEFT, BOTTOM_LEFT) => {
             // (└, └) => Some(└)
             let boundary_type = BOTTOM_LEFT;
@@ -288,7 +306,7 @@ fn combine_symbols(
                 invisible,
                 color,
             })
-        }
+        },
         (BOTTOM_LEFT, VERTICAL_LEFT) | (BOTTOM_LEFT, HORIZONTAL_DOWN) => {
             // (└, ┤) => Some(┼)
             // (└, ┬) => Some(┼)
@@ -298,7 +316,7 @@ fn combine_symbols(
                 invisible,
                 color,
             })
-        }
+        },
         (BOTTOM_LEFT, VERTICAL_RIGHT) => {
             // (└, ├) => Some(├)
             let boundary_type = VERTICAL_RIGHT;
@@ -307,7 +325,7 @@ fn combine_symbols(
                 invisible,
                 color,
             })
-        }
+        },
         (BOTTOM_LEFT, HORIZONTAL_UP) => {
             // (└, ┴) => Some(┴)
             let boundary_type = HORIZONTAL_UP;
@@ -316,7 +334,7 @@ fn combine_symbols(
                 invisible,
                 color,
             })
-        }
+        },
         (VERTICAL_LEFT, VERTICAL_LEFT) => {
             // (┤, ┤) => Some(┤)
             let boundary_type = VERTICAL_LEFT;
@@ -325,7 +343,7 @@ fn combine_symbols(
                 invisible,
                 color,
             })
-        }
+        },
         (VERTICAL_LEFT, VERTICAL_RIGHT)
         | (VERTICAL_LEFT, HORIZONTAL_DOWN)
         | (VERTICAL_LEFT, HORIZONTAL_UP) => {
@@ -338,7 +356,7 @@ fn combine_symbols(
                 invisible,
                 color,
             })
-        }
+        },
         (VERTICAL_RIGHT, VERTICAL_RIGHT) => {
             // (├, ├) => Some(├)
             let boundary_type = VERTICAL_RIGHT;
@@ -347,7 +365,7 @@ fn combine_symbols(
                 invisible,
                 color,
             })
-        }
+        },
         (VERTICAL_RIGHT, HORIZONTAL_DOWN) | (VERTICAL_RIGHT, HORIZONTAL_UP) => {
             // (├, ┬) => Some(┼)
             // (├, ┴) => Some(┼)
@@ -357,7 +375,7 @@ fn combine_symbols(
                 invisible,
                 color,
             })
-        }
+        },
         (HORIZONTAL_DOWN, HORIZONTAL_DOWN) => {
             // (┬, ┬) => Some(┬)
             let boundary_type = HORIZONTAL_DOWN;
@@ -366,7 +384,7 @@ fn combine_symbols(
                 invisible,
                 color,
             })
-        }
+        },
         (HORIZONTAL_DOWN, HORIZONTAL_UP) => {
             // (┬, ┴) => Some(┼)
             let boundary_type = CROSS;
@@ -375,7 +393,7 @@ fn combine_symbols(
                 invisible,
                 color,
             })
-        }
+        },
         (HORIZONTAL_UP, HORIZONTAL_UP) => {
             // (┴, ┴) => Some(┴)
             let boundary_type = HORIZONTAL_UP;
@@ -384,7 +402,7 @@ fn combine_symbols(
                 invisible,
                 color,
             })
-        }
+        },
         (_, _) => combine_symbols(next_symbol, current_symbol),
     }
 }
@@ -510,17 +528,16 @@ impl Boundaries {
             }
         }
     }
-    pub fn vte_output(&self) -> String {
-        let mut vte_output = String::new();
+    pub fn render(&self) -> Vec<CharacterChunk> {
+        let mut character_chunks = vec![];
         for (coordinates, boundary_character) in &self.boundary_characters {
-            vte_output.push_str(&format!(
-                "\u{1b}[{};{}H\u{1b}[m{}",
-                coordinates.y + 1,
-                coordinates.x + 1,
-                boundary_character
-            )); // goto row/col + boundary character
+            character_chunks.push(CharacterChunk::new(
+                vec![boundary_character.as_terminal_character()],
+                coordinates.x,
+                coordinates.y,
+            ));
         }
-        vte_output
+        character_chunks
     }
     fn rect_right_boundary_is_before_screen_edge(&self, rect: &dyn Pane) -> bool {
         rect.x() + rect.cols() < self.viewport.cols
