@@ -24,6 +24,10 @@ pub fn set_session_name(v: String) {
     set_var(SESSION_NAME_ENV_KEY, v);
 }
 
+pub fn set_initial_environment_vars() {
+    set_var("COLORTERM", "24bit");
+}
+
 pub const SOCKET_DIR_ENV_KEY: &str = "ZELLIJ_SOCKET_DIR";
 pub fn get_socket_dir() -> Result<String> {
     Ok(var(SOCKET_DIR_ENV_KEY)?)
@@ -36,7 +40,7 @@ pub struct EnvironmentVariablesFromYaml {
 }
 
 impl EnvironmentVariablesFromYaml {
-    /// Merges two structs, keys from `other` supercede keys from `self`
+    /// Merges two structs, keys from `other` supersede keys from `self`
     pub fn merge(&self, other: Self) -> Self {
         let mut env = self.clone();
         env.env.extend(other.env);
