@@ -3,6 +3,7 @@
 
 use crate::channels::{SenderWithContext, ASYNCOPENCALLS, OPENCALLS};
 use colored::*;
+use log::error;
 use serde::{Deserialize, Serialize};
 use std::fmt::{Display, Error, Formatter};
 use std::panic::PanicInfo;
@@ -70,13 +71,15 @@ where
 
     let mut report: Report = Panic(format!("\u{1b}[0;31m{}\u{1b}[0;0m", msg)).into();
 
+    let mut location_string = String::new();
     if let Some(location) = info.location() {
-        report = report.wrap_err(format!(
+        location_string = format!(
             "At {}:{}:{}",
             location.file(),
             location.line(),
             location.column()
-        ));
+        );
+        report = report.wrap_err(location_string.clone());
     }
 
     if !err_ctx.is_empty() {
@@ -88,6 +91,17 @@ where
         thread
     ));
 
+    error!(
+        "{}",
+        format!(
+            "Panic occured:
+             thread: {}
+             location: {}
+             message: {}",
+            thread, location_string, msg
+        )
+    );
+
     if thread == "main" {
         // here we only show the first line because the backtrace is not readable otherwise
         // a better solution would be to escape raw mode before we do this, but it's not trivial
@@ -265,16 +279,19 @@ pub enum ScreenContext {
     SetFixedWidth,
     ClosePane,
     UpdatePaneName,
+    UndoRenamePane,
     NewTab,
     SwitchTabNext,
     SwitchTabPrev,
     CloseTab,
     GoToTab,
     UpdateTabName,
+    UndoRenameTab,
     TerminalResize,
     TerminalPixelDimensions,
     TerminalBackgroundColor,
     TerminalForegroundColor,
+    TerminalColorRegisters,
     ChangeMode,
     LeftClick,
     RightClick,
@@ -288,6 +305,12 @@ pub enum ScreenContext {
     RemoveOverlay,
     ConfirmPrompt,
     DenyPrompt,
+    UpdateSearch,
+    SearchDown,
+    SearchUp,
+    SearchToggleCaseSensitivity,
+    SearchToggleWholeWord,
+    SearchToggleWrap,
 }
 
 /// Stack call representations corresponding to the different types of [`PtyInstruction`]s.
@@ -327,6 +350,7 @@ pub enum ClientContext {
     ServerError,
     SwitchToMode,
     Connected,
+    ActiveClients,
 }
 
 /// Stack call representations corresponding to the different types of [`ServerInstruction`]s.
@@ -342,6 +366,7 @@ pub enum ServerContext {
     DetachSession,
     AttachClient,
     ConnStatus,
+    ActiveClients,
 }
 
 #[derive(Debug, Clone, Copy, PartialEq, Serialize, Deserialize)]
