@@ -16,7 +16,6 @@ use crate::{
     pane_size::{Dimension, PaneGeom},
     setup,
 };
-use crate::{serde, serde_yaml};
 
 use super::{
     config::ConfigFromYaml,
@@ -35,7 +34,6 @@ use std::{fs::File, io::prelude::*};
 use url::Url;
 
 #[derive(Debug, PartialEq, Eq, Serialize, Deserialize, Clone, Copy)]
-#[serde(crate = "self::serde")]
 pub enum Direction {
     #[serde(alias = "horizontal")]
     Horizontal,
@@ -54,17 +52,15 @@ impl Not for Direction {
     }
 }
 
-#[derive(Debug, Serialize, Deserialize, Clone, Copy, PartialEq)]
-#[serde(crate = "self::serde")]
+#[derive(Debug, Serialize, Deserialize, Clone, Copy, PartialEq, Eq)]
 pub enum SplitSize {
     #[serde(alias = "percent")]
-    Percent(f64), // 1 to 100
+    Percent(u64), // 1 to 100
     #[serde(alias = "fixed")]
     Fixed(usize), // An absolute number of columns or rows
 }
 
 #[derive(Debug, Serialize, Deserialize, Clone, PartialEq, Eq)]
-#[serde(crate = "self::serde")]
 pub enum Run {
     #[serde(rename = "plugin")]
     Plugin(RunPlugin),
@@ -73,7 +69,6 @@ pub enum Run {
 }
 
 #[derive(Debug, Serialize, Deserialize, Clone, PartialEq, Eq)]
-#[serde(crate = "self::serde")]
 pub enum RunFromYaml {
     #[serde(rename = "plugin")]
     Plugin(RunPluginFromYaml),
@@ -82,7 +77,6 @@ pub enum RunFromYaml {
 }
 
 #[derive(Debug, Serialize, Deserialize, Clone, PartialEq, Eq)]
-#[serde(crate = "self::serde")]
 pub struct RunPluginFromYaml {
     #[serde(default)]
     pub _allow_exec_host_cmd: bool,
@@ -90,7 +84,6 @@ pub struct RunPluginFromYaml {
 }
 
 #[derive(Debug, Serialize, Deserialize, Clone, PartialEq, Eq)]
-#[serde(crate = "self::serde")]
 pub struct RunPlugin {
     #[serde(default)]
     pub _allow_exec_host_cmd: bool,
@@ -98,7 +91,6 @@ pub struct RunPlugin {
 }
 
 #[derive(Debug, Serialize, Deserialize, Clone, PartialEq, Eq)]
-#[serde(crate = "self::serde")]
 pub enum RunPluginLocation {
     File(PathBuf),
     Zellij(PluginTag),
@@ -133,7 +125,6 @@ impl fmt::Display for RunPluginLocation {
 
 // The layout struct ultimately used to build the layouts.
 #[derive(Debug, Serialize, Deserialize, Clone, PartialEq)]
-#[serde(crate = "self::serde")]
 pub struct Layout {
     pub direction: Direction,
     #[serde(default)]
@@ -152,7 +143,6 @@ pub struct Layout {
 // https://github.com/bincode-org/bincode/issues/245
 // flattened fields don't retain size information.
 #[derive(Debug, Serialize, Deserialize, Clone, PartialEq)]
-#[serde(crate = "self::serde")]
 #[serde(default)]
 pub struct LayoutFromYamlIntermediate {
     #[serde(default)]
@@ -170,7 +160,6 @@ pub struct LayoutFromYamlIntermediate {
 // The struct that is used to deserialize the layout from
 // a yaml configuration file
 #[derive(Debug, Serialize, Deserialize, Clone, PartialEq, Default)]
-#[serde(crate = "self::serde")]
 #[serde(default)]
 pub struct LayoutFromYaml {
     #[serde(default)]
@@ -201,7 +190,7 @@ impl LayoutFromYamlIntermediate {
                     return Ok(LayoutFromYamlIntermediate::default());
                 }
                 return Err(ConfigError::Serde(e));
-            }
+            },
             Ok(config) => config,
         };
 
@@ -211,7 +200,7 @@ impl LayoutFromYamlIntermediate {
                     tab.check()?;
                 }
                 Ok(layout)
-            }
+            },
             None => Ok(LayoutFromYamlIntermediate::default()),
         }
     }
@@ -225,7 +214,7 @@ impl LayoutFromYamlIntermediate {
                     return Ok(LayoutFromYamlIntermediate::default());
                 }
                 return Err(ConfigError::Serde(e));
-            }
+            },
             Ok(config) => config,
         };
         Ok(layout)
@@ -276,7 +265,7 @@ impl LayoutFromYamlIntermediate {
                 } else {
                     LayoutFromYamlIntermediate::from_default_assets(layout)
                 }
-            }
+            },
             None => LayoutFromYamlIntermediate::from_default_assets(layout),
         }
     }
@@ -341,7 +330,7 @@ impl LayoutFromYaml {
                     return Ok(LayoutFromYaml::default());
                 }
                 return Err(ConfigError::Serde(e));
-            }
+            },
             Ok(config) => config,
         };
 
@@ -351,7 +340,7 @@ impl LayoutFromYaml {
                     tab.check()?;
                 }
                 Ok(layout)
-            }
+            },
             None => Ok(LayoutFromYaml::default()),
         }
     }
@@ -362,7 +351,7 @@ impl LayoutFromYaml {
         match layout_dir {
             Some(dir) => {
                 Self::new(&dir.join(layout)).or_else(|_| Self::from_default_assets(layout))
-            }
+            },
             None => Self::from_default_assets(layout),
         }
     }
@@ -422,7 +411,6 @@ impl LayoutFromYaml {
 // The struct that is used to deserialize the session from
 // a yaml configuration file
 #[derive(Debug, Default, Serialize, Deserialize, Clone, PartialEq)]
-#[serde(crate = "self::serde")]
 pub struct SessionFromYaml {
     pub name: Option<String>,
     #[serde(default = "default_as_some_true")]
@@ -436,7 +424,6 @@ fn default_as_some_true() -> Option<bool> {
 // The struct that carries the information template that is used to
 // construct the layout
 #[derive(Debug, Serialize, Deserialize, Clone, PartialEq)]
-#[serde(crate = "self::serde")]
 pub struct LayoutTemplate {
     pub direction: Direction,
     #[serde(default)]
@@ -453,7 +440,7 @@ pub struct LayoutTemplate {
 }
 
 impl LayoutTemplate {
-    // Insert an optional `[TabLayout]` at the correct postion
+    // Insert an optional `[TabLayout]` at the correct position
     pub fn insert_tab_layout(mut self, tab_layout: Option<TabLayout>) -> Self {
         if self.body {
             return tab_layout.unwrap_or_default().into();
@@ -481,8 +468,7 @@ impl LayoutTemplate {
 }
 
 // The tab-layout struct used to specify each individual tab.
-#[derive(Debug, Serialize, Deserialize, Clone, PartialEq)]
-#[serde(crate = "self::serde")]
+#[derive(Debug, Serialize, Deserialize, Clone, PartialEq, Eq)]
 pub struct TabLayout {
     #[serde(default)]
     pub direction: Direction,
@@ -518,8 +504,8 @@ impl Layout {
             match part.run {
                 Some(Run::Command(_)) | None => {
                     total_panes += part.total_terminal_panes();
-                }
-                Some(Run::Plugin(_)) => {}
+                },
+                Some(Run::Plugin(_)) => {},
             }
         }
         total_panes
@@ -606,7 +592,7 @@ fn split_space(space_to_split: &PaneGeom, layout: &Layout) -> Vec<(Layout, PaneG
 
     for (&size, part) in sizes.iter().zip(&layout.parts) {
         let split_dimension = match size {
-            Some(SplitSize::Percent(percent)) => Dimension::percent(percent),
+            Some(SplitSize::Percent(percent)) => Dimension::percent(percent as f64),
             Some(SplitSize::Fixed(size)) => Dimension::fixed(size),
             None => {
                 let free_percent = if let Some(p) = split_dimension_space.as_percent() {
@@ -614,7 +600,7 @@ fn split_space(space_to_split: &PaneGeom, layout: &Layout) -> Vec<(Layout, PaneG
                         .iter()
                         .map(|&s| {
                             if let Some(SplitSize::Percent(ip)) = s {
-                                ip
+                                ip as f64
                             } else {
                                 0.0
                             }
@@ -624,7 +610,7 @@ fn split_space(space_to_split: &PaneGeom, layout: &Layout) -> Vec<(Layout, PaneG
                     panic!("Implicit sizing within fixed-size panes is not supported");
                 };
                 Dimension::percent(free_percent / flex_parts as f64)
-            }
+            },
         };
         inherited_dimension.set_inner(
             layout
@@ -682,7 +668,7 @@ impl TryFrom<Url> for RunPluginLocation {
                         Err(_) => Err(PluginsConfigError::InvalidPluginLocation(path.to_owned())),
                     })
                     .map(Self::File)
-            }
+            },
             _ => Err(PluginsConfigError::InvalidUrl(url)),
         }
     }
