@@ -1,12 +1,16 @@
 //! IPC stuff for starting to split things into a client and server model.
-
 use crate::{
     cli::CliArgs,
+    data::{ClientId, InputMode, Style},
     errors::{get_current_ctx, ErrorContext},
-    input::{actions::Action, layout::LayoutFromYaml, options::Options, plugins::PluginsConfig},
+    input::{
+        actions::Action, keybinds::Keybinds, layout::LayoutFromYaml, options::Options,
+        plugins::PluginsConfig,
+    },
     pane_size::{Size, SizeInPixels},
 };
 use interprocess::local_socket::LocalSocketStream;
+use log::warn;
 use nix::unistd::dup;
 use serde::{Deserialize, Serialize};
 use std::{
@@ -16,8 +20,6 @@ use std::{
     os::unix::io::{AsRawFd, FromRawFd},
 };
 
-use zellij_tile::{data::InputMode, prelude::Style};
-
 type SessionId = u64;
 
 #[derive(PartialEq, Eq, Serialize, Deserialize, Hash)]
@@ -37,10 +39,11 @@ pub enum ClientType {
     Writer,
 }
 
-#[derive(Default, Serialize, Deserialize, Debug, Clone, Copy)]
+#[derive(Default, Serialize, Deserialize, Debug, Clone)]
 pub struct ClientAttributes {
     pub size: Size,
     pub style: Style,
+    pub keybinds: Keybinds,
 }
 
 #[derive(Default, Serialize, Deserialize, Debug, Clone, Copy, PartialEq, Eq)]
@@ -74,9 +77,11 @@ pub enum ClientToServerMsg {
     DetachSession(SessionId),
     // Disconnect from the session we're connected to
     DisconnectFromSession,*/
+    DetachSession(Vec<ClientId>),
     TerminalPixelDimensions(PixelDimensions),
     BackgroundColor(String),
     ForegroundColor(String),
+    ColorRegisters(Vec<(usize, String)>),
     TerminalResize(Size),
     NewClient(
         ClientAttributes,
@@ -86,10 +91,11 @@ pub enum ClientToServerMsg {
         Option<PluginsConfig>,
     ),
     AttachClient(ClientAttributes, Options),
-    Action(Action),
+    Action(Action, Option<ClientId>),
     ClientExited,
     KillSession,
     ConnStatus,
+    ListClients,
 }
 
 // Types of messages sent from the server to the client
@@ -104,11 +110,13 @@ pub enum ServerToClientMsg {
     Exit(ExitReason),
     SwitchToMode(InputMode),
     Connected,
+    ActiveClients(Vec<ClientId>),
 }
 
 #[derive(Serialize, Deserialize, Debug, Clone)]
 pub enum ExitReason {
     Normal,
+    NormalDetached,
     ForceDetached,
     CannotAttach,
     Error(String),
@@ -118,6 +126,7 @@ impl Display for ExitReason {
     fn fmt(&self, f: &mut Formatter) -> Result<(), Error> {
         match self {
             Self::Normal => write!(f, "Bye from Zellij!"),
+            Self::NormalDetached => write!(f, "Session detached"),
             Self::ForceDetached => write!(
                 f,
                 "Session was detached from this client (possibly because another client connected)"
@@ -149,7 +158,7 @@ impl<T: Serialize> IpcSenderWithContext<T> {
     /// Sends an event, along with the current [`ErrorContext`], on this [`IpcSenderWithContext`]'s socket.
     pub fn send(&mut self, msg: T) {
         let err_ctx = get_current_ctx();
-        bincode::serialize_into(&mut self.sender, &(msg, err_ctx)).unwrap();
+        rmp_serde::encode::write(&mut self.sender, &(msg, err_ctx)).unwrap();
         // TODO: unwrapping here can cause issues when the server disconnects which we don't mind
         // do we need to handle errors here in other cases?
         let _ = self.sender.flush();
@@ -187,9 +196,12 @@ where
 
     /// Receives an event, along with the current [`ErrorContext`], on this [`IpcReceiverWithContext`]'s socket.
     pub fn recv(&mut self) -> Option<(T, ErrorContext)> {
-        match bincode::deserialize_from(&mut self.receiver) {
+        match rmp_serde::decode::from_read(&mut self.receiver) {
             Ok(msg) => Some(msg),
-            Err(_) => None,
+            Err(e) => {
+                warn!("Error in IpcReceiver.recv(): {:?}", e);
+                None
+            },
         }
     }
 
